@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgis.h 12607 2009-12-23 22:55:10Z jef $ */
+/* $Id: qgis.h 14172 2010-09-01 08:48:55Z mhugent $ */
 
 #ifndef QGIS_H
 #define QGIS_H
@@ -46,26 +46,28 @@ class CORE_EXPORT QGis
     // Feature types
     enum WkbType
     {
+      WKBUnknown = 0,
       WKBPoint = 1,
       WKBLineString,
       WKBPolygon,
       WKBMultiPoint,
       WKBMultiLineString,
       WKBMultiPolygon,
-      WKBUnknown,
+      WKBNoGeometry = 100, //attributes only
       WKBPoint25D = 0x80000001,
       WKBLineString25D,
       WKBPolygon25D,
       WKBMultiPoint25D,
       WKBMultiLineString25D,
-      WKBMultiPolygon25D
+      WKBMultiPolygon25D,
     };
     enum GeometryType
     {
       Point,
       Line,
       Polygon,
-      UnknownGeometry
+      UnknownGeometry,
+      NoGeometry
     };
 
     // String representation of geometry types (set in qgis.cpp)
@@ -147,8 +149,10 @@ const QString GEOPROJ4 = "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs";
 const long GEOSRID = 4326;
 /** Magic number for a geographic coord sys in QGIS srs.db tbl_srs.srs_id */
 const long GEOCRS_ID = 3452;
-/**  Magic number for a geographic coord sys in EpsgCrsId ID format */
+/** Magic number for a geographic coord sys in EpsgCrsId ID format */
 const long GEO_EPSG_CRS_ID = 4326;
+/** Geographic coord sys from EPSG authority */
+const QString GEO_EPSG_CRS_AUTHID = "EPSG:4326";
 /** The length of the string "+proj=" */
 const int PROJ_PREFIX_LEN = 6;
 /** The length of the string "+ellps=" */
