@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgisinterface.h 12372 2009-12-08 10:35:07Z mhugent $ */
+/* $Id: qgisinterface.h 14452 2010-10-30 09:33:01Z wonder $ */
 #ifndef QGISINTERFACE_H
 #define QGISINTERFACE_H
 
@@ -31,13 +31,13 @@ class QWidget;
 #include <map>
 
 
-class QgisApp;
 class QgsComposerView;
 class QgsMapLayer;
 class QgsMapCanvas;
 class QgsRasterLayer;
 class QgsVectorLayer;
 class QgsLegendInterface;
+class QgsFeature;
 
 /** \ingroup gui
  * QgisInterface
@@ -135,9 +135,16 @@ class GUI_EXPORT QgisInterface : public QObject
     /** Remove specified dock widget from main window (doesn't delete it). Added in QGIS 1.1. */
     virtual void removeDockWidget( QDockWidget * dockwidget ) = 0;
 
-    /** refresh the legend of a layer */
+    /** refresh the legend of a layer
+     \note deprecated - use QgsLegendInterface::refreshLayerSymbology
+     */
     virtual void refreshLegend( QgsMapLayer *l ) = 0;
 
+    /** open layer properties dialog
+     \note added in 1.5
+     */
+    virtual void showLayerProperties( QgsMapLayer *l ) = 0;
+
     /** Add window to Window menu. The action title is the window title
      * and the action should raise, unminimize and activate the window. */
     virtual void addWindow( QAction *action ) = 0;
@@ -156,13 +163,13 @@ class GUI_EXPORT QgisInterface : public QObject
     */
     virtual bool unregisterMainWindowAction( QAction* action ) = 0;
 
-    // TODO: is this deprecated in favour of QgsContextHelp?
+    // @todo is this deprecated in favour of QgsContextHelp?
     /** Open a url in the users browser. By default the QGIS doc directory is used
      * as the base for the URL. To open a URL that is not relative to the installed
      * QGIS documentation, set useQgisDocDirectory to false.
      * @param url URL to open
      * @param useQgisDocDirectory If true, the URL will be formed by concatenating
-     * url to the QGIS documentation directory path (<prefix>/share/doc)
+     * url to the QGIS documentation directory path (prefix/share/doc)
      */
     virtual void openURL( QString url, bool useQgisDocDirectory = true ) = 0;
 
@@ -187,6 +194,7 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QToolBar *layerToolBar() = 0;
     virtual QToolBar *mapNavToolToolBar() = 0;
     virtual QToolBar *digitizeToolBar() = 0;
+    virtual QToolBar *advancedDigitizeToolBar() = 0;  // added in v1.5
     virtual QToolBar *attributesToolBar() = 0;
     virtual QToolBar *pluginToolBar() = 0;
     virtual QToolBar *helpToolBar() = 0;
@@ -221,6 +229,10 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QAction *actionMoveVertex() = 0;
     virtual QAction *actionAddRing() = 0;
     virtual QAction *actionAddIsland() = 0;
+    virtual QAction *actionSimplifyFeature() = 0;
+    virtual QAction *actionDeleteRing() = 0;
+    virtual QAction *actionDeletePart() = 0;
+    virtual QAction *actionNodeTool() = 0;
     virtual QAction *actionEditSeparator2() = 0;
 
     //! View menu actions
@@ -228,6 +240,10 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QAction *actionZoomIn() = 0;
     virtual QAction *actionZoomOut() = 0;
     virtual QAction *actionSelect() = 0;
+    virtual QAction *actionSelectRectangle() = 0;
+    virtual QAction *actionSelectPolygon() = 0;
+    virtual QAction *actionSelectFreehand() = 0;
+    virtual QAction *actionSelectRadius() = 0;
     virtual QAction *actionIdentify() = 0;
     virtual QAction *actionMeasure() = 0;
     virtual QAction *actionMeasureArea() = 0;
@@ -287,6 +303,11 @@ class GUI_EXPORT QgisInterface : public QObject
     virtual QAction *actionHelpSeparator2() = 0;
     virtual QAction *actionAbout() = 0;
 
+    //! Open feature form
+    // returns true when dialog was accepted
+    // @added in 1.6
+    virtual bool openFeatureForm( QgsVectorLayer *vlayer, QgsFeature &f, bool updateFeatureOnly = false ) = 0;
+
   signals:
     /** Emited whenever current (selected) layer changes.
      *  The pointer to layer can be null if no layer is selected
@@ -298,6 +319,30 @@ class GUI_EXPORT QgisInterface : public QObject
     /**This signal is emitted before a new composer instance is going to be removed
        @note added in version 1.4*/
     void composerWillBeRemoved( QgsComposerView* v );
+    /**This signal is emitted when the initialization is complete
+       @note added in version 1.6*/
+    void initializationCompleted();
+    /** emitted when a project file is successfully read
+        @note
+        This is useful for plug-ins that store properties with project files.  A
+        plug-in can connect to this signal.  When it is emitted, the plug-in
+        knows to then check the project properties for any relevant state.
+
+        Added in v1.6
+     */
+    void projectRead();
+    /** emitted when starting an entirely new project
+        @note
+        This is similar to projectRead(); plug-ins might want to be notified
+        that they're in a new project.  Yes, projectRead() could have been
+        overloaded to be used in the case of new projects instead.  However,
+        it's probably more semantically correct to have an entirely separate
+        signal for when this happens.
+
+        Added in v1.6
+      */
+     void newProjectCreated();
+
 };
 
 // FIXME: also in core/qgis.h
