@@ -12,7 +12,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsapplication.h 12528 2009-12-20 12:29:07Z jef $ */
+/* $Id: qgsapplication.h 14479 2010-11-01 12:31:53Z jef $ */
 #ifndef QGSAPPLICATION_H
 #define QGSAPPLICATION_H
 
@@ -26,7 +26,8 @@ class CORE_EXPORT QgsApplication: public QApplication
 {
     Q_OBJECT
   public:
-    QgsApplication( int & argc, char ** argv, bool GUIenabled );
+    //! @note customConfigDir parameter added in v1.6
+    QgsApplication( int & argc, char ** argv, bool GUIenabled, QString customConfigPath = QString() );
     virtual ~QgsApplication();
 
     //! Catch exceptions when sending event to receiver.
@@ -37,7 +38,7 @@ class CORE_EXPORT QgsApplication: public QApplication
      * The theme search path usually will be pkgDataPath + "/themes/" + themName + "/"
      * but plugin writers etc can use themeName() as a basis for searching
      * for resources in their own datastores e.g. a Qt4 resource bundle.
-     * @Note A basic test will be carried out to ensure the theme search path
+     * @note A basic test will be carried out to ensure the theme search path
      * based on the supplied theme name exists. If it does not the theme name will
      * be reverted to 'default'.
      */
@@ -89,6 +90,10 @@ class CORE_EXPORT QgsApplication: public QApplication
     //! Returns the path to the master qgis.db file.
     static const QString qgisMasterDbFilePath();
 
+    //! Returns the path to the spatialite template db file.
+    //! @note added in 1.5
+    static const QString qgisSpatialiteDbTemplatePath();
+
     //! Returns the path to the settings directory in user's home dir
     static const QString qgisSettingsDirPath();
 
@@ -127,6 +132,11 @@ class CORE_EXPORT QgsApplication: public QApplication
     //! Returns the path to the default theme directory.
     static const QString defaultThemePath();
 
+    //! Returns path to the desired icon file.
+    //! First it tries to use the active theme path, then default theme path
+    //! @note Added in 1.5
+    static QString iconPath( QString iconFile );
+
     //! Returns the path to user's style. Added in QGIS 1.4
     static const QString userStyleV2Path();
 
@@ -134,7 +144,7 @@ class CORE_EXPORT QgsApplication: public QApplication
     static const QString defaultStyleV2Path();
 
     //! Alters prefix path - used by 3rd party apps
-    static void setPrefixPath( const QString thePrefixPath, bool useDefaultPaths = FALSE );
+    static void setPrefixPath( const QString thePrefixPath, bool useDefaultPaths = false );
 
     //! Alters plugin path - used by 3rd party apps
     static void setPluginPath( const QString thePluginPath );
@@ -142,6 +152,9 @@ class CORE_EXPORT QgsApplication: public QApplication
     //! Alters pkg data path - used by 3rd party apps
     static void setPkgDataPath( const QString thePkgDataPath );
 
+    //! Alters default svg paths - used by 3rd party apps. Added in QGIS 1.5
+    static void setDefaultSvgPaths( const QStringList& pathList );
+
     //! loads providers
     static void initQgis();
 
@@ -179,11 +192,21 @@ class CORE_EXPORT QgsApplication: public QApplication
      */
     static void registerOgrDrivers();
 
+    /**Converts absolute path to path relative to target
+      @note: this method was added in version 1.6*/
+    static QString absolutePathToRelativePath( QString apath, QString targetPath );
+    /**Converts path relative to target to an absolute path
+      @note: this method was added in version 1.6*/
+    static QString relativePathToAbsolutePath( QString rpath, QString targetPath );
+
   private:
     static QString mPrefixPath;
     static QString mPluginPath;
     static QString mPkgDataPath;
     static QString mThemeName;
+    static QStringList mDefaultSvgPaths;
+
+    static QString mConfigPath;
 };
 
 #endif
