@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgscomposerview.h 12248 2009-11-25 10:17:28Z mhugent $ */
+/* $Id: qgscomposerview.h 13316 2010-04-15 14:57:32Z mhugent $ */
 #ifndef QGSCOMPOSERVIEW_H
 #define QGSCOMPOSERVIEW_H
 
@@ -32,6 +32,7 @@ class QgsComposerMap;
 class QgsComposerPicture;
 class QgsComposerScaleBar;
 class QgsComposerShape;
+class QgsComposerAttributeTable;
 
 /** \ingroup MapComposer
  * \ingroup gui
@@ -57,6 +58,7 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
       AddScalebar,     // add scalebar
       AddPicture,       // add raster/vector picture
       AddShape, //add shape item (ellipse, rectangle, triangle)
+      AddTable, //add attribute table
       MoveItemContent //move content of item (e.g. content of map)
     };
 
@@ -88,8 +90,10 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
     void addComposerLegend( QgsComposerLegend* legend );
     /**Adds picture to the graphics scene and advices composer to create a widget for it (through signal)*/
     void addComposerPicture( QgsComposerPicture* picture );
-    /**Adds a composer shape to the graphics scene and acvices composer to create a widget for it (through signal)*/
+    /**Adds a composer shape to the graphics scene and advices composer to create a widget for it (through signal)*/
     void addComposerShape( QgsComposerShape* shape );
+    /**Adds a composer table to the graphics scene and advices composer to create a widget for it (through signal)*/
+    void addComposerTable( QgsComposerAttributeTable* table );
 
     /**Returns the composer main window*/
     QMainWindow* composerWindow();
@@ -98,6 +102,7 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
     void mousePressEvent( QMouseEvent* );
     void mouseReleaseEvent( QMouseEvent* );
     void mouseMoveEvent( QMouseEvent* );
+    void mouseDoubleClickEvent( QMouseEvent* e );
 
     void keyPressEvent( QKeyEvent * e );
     void keyReleaseEvent( QKeyEvent * e );
@@ -141,6 +146,8 @@ class GUI_EXPORT QgsComposerView: public QGraphicsView
     void composerPictureAdded( QgsComposerPicture* picture );
     /**Is emitted when a new composer shape has been added*/
     void composerShapeAdded( QgsComposerShape* shape );
+    /**Is emitted when a new composer table has been added*/
+    void composerTableAdded( QgsComposerAttributeTable* table );
     /**Is emitted when a composer item has been removed from the scene*/
     void itemRemoved( QgsComposerItem* );
     /**Current action (e.g. adding composer map) has been finished. The purpose of this signal is that
