@@ -8,26 +8,29 @@
 // <int>.<int>.<int>-<any text>.
 // or else upgrading old project file will not work
 // reliably.
-#define VERSION "1.4.0-Enceladus"
+#define VERSION "1.6.0-Copiapo"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to 
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 140
-#define VERSION_INT 10400
+//#define VERSION_INT 160
+#define VERSION_INT 10600
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Enceladus"
+#define RELEASE_NAME "Copiapo"
 
-#define QGIS_PLUGIN_SUBDIR "lib/qgis"
+#define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
+#define QGIS_LIBEXEC_SUBDIR "lib/qgis"
+#define QGIS_LIB_SUBDIR "lib"
+#define CMAKE_INSTALL_PREFIX "/usr/local"
 
 #define HAVE_POSTGRESQL
 
 #define HAVE_SPATIALITE
 
-#define HAVE_PYTHON
+/* #undef HAVE_PYTHON */
 
 #endif
 
