@@ -48,7 +48,7 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     {
       InternalCrsId,
       PostgisCrsId,
-      EpsgCrsId
+      EpsgCrsId  // deprecated
     };
 
     //! Default constructor
@@ -88,7 +88,7 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
      *
      * \note This function only deals with EpsgCrsId labels only at this time.
      *
-     * \retval FALSE if not given an EpsgCrsId label
+     * \retval false if not given an EpsgCrsId label
      */
     bool createFromOgcWmsCrs( QString theCrs );
 
@@ -152,7 +152,7 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
      *   or isSameGeocs (essentially calling the == overloaded operator). We'll try to
      *   be smart about this and first parse out the proj and ellpse strings and only
      *   check for a match in entities that have the same ellps and proj entries so
-     *   that it doesnt munch yer cpu so much.
+     *   that it doesn't munch yer cpu so much.
      *
      * @note If the srs was not matched, we will create a new entry on the users tbl_srs
      *    for this srs.
@@ -179,13 +179,13 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     /*! This is a globbing function to try to find a record in the database
      *  that matches a CRS defined only by a proj4string. The goal is to
      *  learn what the tbl_srs.srs_id value is for the CRS. Internally
-     *  the source CRS is converted to and OGR srs object using the proj4string
+     *  the source CRS is converted to an OGR srs object using the proj4string call
      *  and then every record in the database that matches projection and ellipsoid
      *  will be converted to an OGR srs in turn and compared to the source CRS.
      *  There are some gotchas with using ogr isSame() srs comparison, but
      *  its more effective than using straight string comparison of proj4params.
      *  @note The ellipsoid and projection acronyms must be set as well as the proj4string!
-     *  @return lomg the SrsId of the matched CRS
+     *  @return long the SrsId of the matched CRS
      */
     long findMatchingProj();
 
@@ -211,17 +211,19 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     bool readXML( QDomNode & theNode );
     /*! Stores state to the given Dom node in the given document.
      * Below is an example of the generated tag.
-     *  <spatialrefsys>
-     *      <proj4>+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs </proj4>
-     *       <srsid>2585</srsid>
-     *       <srid>4326</srid>
-     *       <epsg>4326</epsg>
-     *       <description>WGS 84</description>
-     *       <projectionacronym>longlat</projectionacronym>
-     *       <ellipsoidacronym>WGS84</ellipsoidacronym>
-     *   </spatialrefsys>
+     \verbatim
+      <spatialrefsys>
+          <proj4>+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs </proj4>
+           <srsid>2585</srsid>
+           <srid>4326</srid>
+           <epsg>4326</epsg>
+           <description>WGS 84</description>
+           <projectionacronym>longlat</projectionacronym>
+           <ellipsoidacronym>WGS84</ellipsoidacronym>
+       </spatialrefsys>
+     \endverbatim
      * @param theNode The node in which state will be restored
-     * @param theDom The document in which state will be stored
+     * @param theDoc The document in which state will be stored
      * @return bool True on success, False on failure
      */
     bool writeXML( QDomNode & theNode, QDomDocument & theDoc ) const;
@@ -249,10 +251,16 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     long postgisSrid() const;
 
     /*! Get the EpsgCrsId identifier for this srs
-     * @return  long theEpsg the ESPG identifier for this srs (defaults to 0)
+     * @return  long theEpsg the EPSG identifier for this srs (defaults to 0)
      */
     long epsg() const;
 
+    /*! Get the authority identifier for this srs
+     * @return  QString the Authority identifier for this srs
+     * @note added in 1.5
+     */
+    QString authid() const;
+
     /*! Get the Description
      * @return  QString the Description A textual description of the srs.
      * @note A zero length string will be returned if the description is uninitialised
@@ -309,21 +317,21 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     // a fully valid crs. Programmers should use the createFrom* methods rather
   private:
     /** A static helper function to find out the proj4 string for a srsid
-      * @param int theSrsId The srsid used for the lookup
+      * @param theSrsId The srsid used for the lookup
       * @return QString The proj4 string
     */
     static QString proj4FromSrsId( const int theSrsId );
 
     /*! Set the QGIS  SrsId
-     *  @param  long theSrsId The internal sqlite3 srs.db primary key for this srs
+     *  @param theSrsId The internal sqlite3 srs.db primary key for this srs
      */
     void setInternalId( long theSrsId );
     /*! Set the postgis srid
-     *  @param  long theSrsId The postgis spatial_ref_sys key for this srs
+     *  @param theSrid The postgis spatial_ref_sys key for this srs
      */
     void setSrid( long theSrid );
     /*! Set the Description
-     * @param  QString the Description A textual description of the srs.
+     * @param theDescription A textual description of the srs.
      */
     void setDescription( QString theDescription );
     /* Set the Proj Proj4String.
@@ -331,19 +339,25 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
      */
     void setProj4String( QString theProj4String );
     /*! Set this Geographic? flag
-     * @param  bool theGeoFlag Whether this is a geographic or projected coordinate system
+     * @param theGeoFlag Whether this is a geographic or projected coordinate system
      */
     void setGeographicFlag( bool theGeoFlag );
+
     /*! Set the EpsgCrsId identifier for this srs
-     * @param  long theEpsg the ESPG identifier for this srs (defaults to 0)
+     * @param theEpsg the ESPG identifier for this srs (defaults to 0)
      */
     void setEpsg( long theEpsg );
+
+    /*! Set the authority identifier for this srs
+     * @param theID the authority identifier for this srs (defaults to 0)
+     */
+    void setAuthId( QString theID );
     /*! Set the projection acronym
-     * @param QString the acronym (must be a valid proj4 projection acronym)
+     * @param theProjectionAcronym the acronym (must be a valid proj4 projection acronym)
      */
     void setProjectionAcronym( QString theProjectionAcronym );
     /*! Set the ellipsoid acronym
-     * @param QString the acronym (must be a valid proj4 ellipsoid acronym)
+     * @param theEllipsoidAcronym the acronym (must be a valid proj4 ellipsoid acronym)
      */
     void setEllipsoidAcronym( QString theEllipsoidAcronym );
 
@@ -356,8 +370,8 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     /*! Get a record from the srs.db or qgis.db backends, given an sql statment.
      * @note only handles queries that return a single record.
      * @note it will first try the system srs.db then the users qgis.db!
-     * @param QString The sql query to execute
-     * @return QMap An associative array of field name <-> value pairs
+     * @param theSql The sql query to execute
+     * @return An associative array of field name <-> value pairs
      */
     RecordMap getRecord( QString theSql );
 
@@ -379,8 +393,8 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     QGis::UnitType mMapUnits;
     //!If available, the Postgis spatial_ref_sys identifier for this srs (defaults to 0)
     long    mSRID;
-    //!If available the ESPG identifier for this srs (defaults to 0)
-    long    mEpsg ;
+    //!If available the authority identifier for this srs
+    QString mAuthId;
     //! Wehter this srs is properly defined and valid
     bool mIsValidFlag;
 
@@ -393,12 +407,12 @@ class CORE_EXPORT QgsCoordinateReferenceSystem
     //! Helper for getting number of user CRS already in db
     long getRecordCount();
 
-    //! Helper for sql-safin strings
-    const QString sqlSafeString( const QString theSQL );
+    //! Helper for sql-safe value quoting
+    QString quotedValue( QString value );
 
     void *mCRS;
 
-    bool loadFromDb( QString db, QString field, long id );
+    bool loadFromDb( QString db, QString expression, QString value );
 
     QString mValidationHint;
 
