@@ -12,7 +12,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsdistancearea.h 11474 2009-08-21 23:39:11Z homann $ */
+/* $Id: qgsdistancearea.h 14191 2010-09-04 13:08:17Z mhugent $ */
 
 #ifndef QGSDISTANCEAREA_H
 #define QGSDISTANCEAREA_H
@@ -85,7 +85,7 @@ class CORE_EXPORT QgsDistanceArea
     //! compute bearing - in radians
     double bearing( const QgsPoint& p1, const QgsPoint& p2 );
 
-    static QString textUnit( double value, int decimals, QGis::UnitType u, bool isArea );
+    static QString textUnit( double value, int decimals, QGis::UnitType u, bool isArea, bool keepBaseUnit = false );
 
   protected:
 
