@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsgenericprojectionselector.h 11579 2009-09-06 22:08:21Z jef $ */
+/* $Id: qgsgenericprojectionselector.h 13093 2010-03-20 08:54:33Z jef $ */
 #ifndef QGSGENERICPROJECTIONSELECTOR_H
 #define QGSGENERICPROJECTIONSELECTOR_H
 #include <ui_qgsgenericprojectionselectorbase.h>
@@ -63,10 +63,12 @@ class GUI_EXPORT QgsGenericProjectionSelector : public QDialog, private Ui::QgsG
     void setMessage( QString theMessage = "" );
     QString selectedProj4String();
     long selectedCrsId();
+    QString selectedAuthId();
     long selectedEpsg();
 
     void setSelectedCrsName( QString theName );
     void setSelectedCrsId( long theID );
+    void setSelectedAuthId( QString authId );
     void setSelectedEpsg( long theID );
 
     /**
@@ -79,8 +81,6 @@ class GUI_EXPORT QgsGenericProjectionSelector : public QDialog, private Ui::QgsG
      *                  list of projections by.  This is useful in (e.g.) WMS situations
      *                  where you just want to offer what the WMS server can support.
      *
-     * \note This function only deals with EpsgCrsId labels only at this time.
-     *
      * \warning This function's behaviour is undefined if it is called after the dialog is shown.
      */
     void setOgcWmsCrsFilter( QSet<QString> crsFilter );
