@@ -12,7 +12,7 @@ email                : morb at ozemail dot com dot au
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsgeometry.h 12528 2009-12-20 12:29:07Z jef $ */
+/* $Id: qgsgeometry.h 14462 2010-10-31 16:41:54Z jef $ */
 
 #ifndef QGSGEOMETRY_H
 #define QGSGEOMETRY_H
@@ -131,6 +131,30 @@ class CORE_EXPORT QgsGeometry
     /** Returns true if wkb of the geometry is of WKBMulti* type */
     bool isMultipart();
 
+    /** compare geometries using GEOS
+      @note added in 1.5
+     */
+    bool isGeosEqual( QgsGeometry & );
+
+    /** check validity using GEOS
+      @note added in 1.5
+     */
+    bool isGeosValid();
+
+    /** check if geometry is empty using GEOS
+      @note added in 1.5
+     */
+    bool isGeosEmpty();
+
+    /** get area of geometry using GEOS
+      @note added in 1.5
+     */
+    double area();
+
+    /** get length of geometry using GEOS
+      @note added in 1.5
+     */
+    double length();
 
     double distance( QgsGeometry& geom );
 
@@ -161,7 +185,7 @@ class CORE_EXPORT QgsGeometry
      *  If the requested vertex number (beforeVertex.back()) is greater
      *  than the last actual vertex on the requested ring and item,
      *  it is assumed that the vertex is to be appended instead of inserted.
-     *  Returns FALSE if atVertex does not correspond to a valid vertex
+     *  Returns false if atVertex does not correspond to a valid vertex
      *  on this geometry (including if this geometry is a Point).
      *  It is up to the caller to distinguish between
      *  these error conditions.  (Or maybe we add another method to this
@@ -172,14 +196,14 @@ class CORE_EXPORT QgsGeometry
     /** Moves the vertex at the given position number
      *  and item (first number is index 0)
      *  to the given coordinates.
-     *  Returns FALSE if atVertex does not correspond to a valid vertex
+     *  Returns false if atVertex does not correspond to a valid vertex
      *  on this geometry
      */
     bool moveVertex( double x, double y, int atVertex );
 
     /** Deletes the vertex at the given position number and item
      *  (first number is index 0)
-     *  Returns FALSE if atVertex does not correspond to a valid vertex
+     *  Returns false if atVertex does not correspond to a valid vertex
      *  on this geometry (including if this geometry is a Point),
      *  or if the number of remaining verticies in the linestring
      *  would be less than two.
@@ -243,14 +267,14 @@ class CORE_EXPORT QgsGeometry
     /**Splits this geometry according to a given line. Note that the geometry is only split once. If there are several intersections
      between geometry and splitLine, only the first one is considered.
     @param splitLine the line that splits the geometry
-    @param newGeometrys OUT: list of new geometries that have been created with the split
+    @param[out] newGeometries list of new geometries that have been created with the split
     @param topological true if topological editing is enabled
-    @topologyTestPoints OUT: points that need to be tested for topological completeness in the dataset
+    @param[out] topologyTestPoints points that need to be tested for topological completeness in the dataset
     @return 0 in case of success, 1 if geometry has not been split, error else*/
     int splitGeometry( const QList<QgsPoint>& splitLine,
                        QList<QgsGeometry*>&newGeometries,
                        bool topological,
-                       QList<QgsPoint>& topologyTestPoints );
+                       QList<QgsPoint> &topologyTestPoints );
 
     /**Replaces a part of this geometry with another line
       @return 0 in case of success
@@ -267,12 +291,41 @@ class CORE_EXPORT QgsGeometry
 
     /** Test for intersection with a rectangle (uses GEOS) */
     bool intersects( const QgsRectangle& r );
-    /** Test for intersection with a geoemetry (uses GEOS) */
+
+    /** Test for intersection with a geometry (uses GEOS) */
     bool intersects( QgsGeometry* geometry );
 
     /** Test for containment of a point (uses GEOS) */
     bool contains( QgsPoint* p );
 
+    /** Test for if geometry is contained in an other (uses GEOS)
+     *  @note added in 1.5 */
+    bool contains( QgsGeometry* geometry );
+
+    /** Test for if geometry is disjoint of an other (uses GEOS)
+     *  @note added in 1.5 */
+    bool disjoint( QgsGeometry* geometry );
+
+    /** Test for if geometry equals an other (uses GEOS)
+     *  @note added in 1.5 */
+    bool equals( QgsGeometry* geometry );
+
+    /** Test for if geometry touch an other (uses GEOS)
+     *  @note added in 1.5 */
+    bool touches( QgsGeometry* geometry );
+
+    /** Test for if geometry overlaps an other (uses GEOS)
+     *  @note added in 1.5 */
+    bool overlaps( QgsGeometry* geometry );
+
+    /** Test for if geometry is within an other (uses GEOS)
+     *  @note added in 1.5 */
+    bool within( QgsGeometry* geometry );
+
+    /** Test for if geometry crosses an other (uses GEOS)
+     *  @note added in 1.5 */
+    bool crosses( QgsGeometry* geometry );
+
     /** Returns a buffer region around this geometry having the given width and with a specified number
         of segments used to approximate curves */
     QgsGeometry* buffer( double distance, int segments );
@@ -339,12 +392,12 @@ class CORE_EXPORT QgsGeometry
 
     /** delete a ring in polygon or multipolygon.
       Ring 0 is outer ring and can't be deleted.
-      @return TRUE on success
+      @return true on success
       @note added in version 1.2 */
     bool deleteRing( int ringNum, int partNum = 0 );
 
     /** delete part identified by the part number
-      @return TRUE on success
+      @return true on success
       @note added in version 1.2 */
     bool deletePart( int partNum );
 
@@ -353,6 +406,37 @@ class CORE_EXPORT QgsGeometry
     @return true in case of success and false else*/
     bool convertToMultiType();
 
+    /** Modifies geometry to avoid intersections with the layers specified in project properties
+     *  @return 0 in case of success,
+     *          1 if geometry is not of polygon type,
+     *          2 if avoid intersection would change the geometry type,
+     *          3 other error during intersection removal
+     *  @note added in 1.5
+     */
+    int avoidIntersections();
+
+
+    class Error
+    {
+        QString message;
+        QgsPoint location;
+        bool hasLocation;
+      public:
+        Error( QString m ) : message( m ), hasLocation( false ) {}
+        Error( QString m, QgsPoint p ) : message( m ), location( p ), hasLocation( true ) {}
+
+        QString what() { return message; };
+        QgsPoint where() { return location; }
+        bool hasWhere() { return hasLocation; }
+    };
+
+    /** Validate geometry and produce a list of geometry errors
+     * @note added in 1.5
+     **/
+    void validateGeometry( QList<Error> &errors );
+
+    static void validatePolyline( QList<Error> &errors, int i, QgsPolyline polyline, bool ring = false );
+    static void validatePolygon( QList<Error> &errors, int i, const QgsPolygon &polygon );
 
   private:
     // Private variables
@@ -382,14 +466,6 @@ class CORE_EXPORT QgsGeometry
 
     // Private functions
 
-    /** Squared distance from point to the given line segment
-     *  TODO: Perhaps move this to QgsPoint
-     */
-    double distanceSquaredPointToSegment( const QgsPoint& point,
-                                          double *x1, double *y1,
-                                          double *x2, double *y2,
-                                          QgsPoint& minDistPoint );
-
     /** Converts from the WKB geometry to the GEOS geometry.
         @return   true in case of success and false else
      */
@@ -405,9 +481,12 @@ class CORE_EXPORT QgsGeometry
      *  If the requested vertex number is greater
      *  than the last actual vertex,
      *  it is assumed that the vertex is to be appended instead of inserted.
+     *  @param x x coordinate
+     *  @param y y coordinate
+     *  @param beforeVertex insert before vertex with this index
      *  @param old_sequence   The sequence to update (The caller remains the owner).
-     *  @param new_sequence   The updated sequence (The caller becomes the owner if the function returns TRUE).
-     *  Returns FALSE if beforeVertex does not correspond to a valid vertex number
+     *  @param new_sequence   The updated sequence (The caller becomes the owner if the function returns true).
+     *  Returns false if beforeVertex does not correspond to a valid vertex number
      *  on the Coordinate Sequence.
      */
     bool insertVertex( double x, double y,
@@ -416,7 +495,6 @@ class CORE_EXPORT QgsGeometry
                        GEOSCoordSequence** new_sequence );
 
     /**Translates a single vertex by dx and dy.
-     @param ptr pointer to the wkb fragment containing the vertex
     @param wkbPosition position in wkb array. Is increased automatically by the function
     @param dx translation of x-coordinate
     @param dy translation of y-coordinate
@@ -424,7 +502,6 @@ class CORE_EXPORT QgsGeometry
     void translateVertex( int& wkbPosition, double dx, double dy, bool hasZValue );
 
     /**Transforms a single vertex by ct.
-     @param ptr pointer to the wkb fragment containing the vertex
     @param wkbPosition position in wkb array. Is increased automatically by the function
     @param ct the QgsCoordinateTransform
     @param hasZValue 25D type?*/
@@ -433,8 +510,8 @@ class CORE_EXPORT QgsGeometry
     //helper functions for geometry splitting
 
     /**Splits line/multiline geometries
-     @splitLine the line that splits the feature
-     @newGeometry new geometry if splitting was successful
+     @param splitLine the line that splits the feature
+     @param newGeometries new geometries if splitting was successful
      @return 0 in case of success, 1 if geometry has not been split, error else*/
     int splitLinearGeometry( GEOSGeometry *splitLine, QList<QgsGeometry*>& newGeometries );
     /**Splits polygon/multipolygon geometries
@@ -472,6 +549,9 @@ class CORE_EXPORT QgsGeometry
       @return 0 not contained, 1 if contained, <0 in case of error*/
     static int pointContainedInLine( const GEOSGeometry* point, const GEOSGeometry* line );
 
+    /**Tests if geom bounding rect is within -180 <= x <= 180, -90 <= y <= 90. Other methods may use more accurate tolerances if this is true*/
+    static bool geomInDegrees( const GEOSGeometry* geom );
+
     /**Returns number of single geometry in a geos geometry. Is save for geos 2 and 3*/
     int numberOfGeometries( GEOSGeometry* g ) const;
 
@@ -486,6 +566,14 @@ class CORE_EXPORT QgsGeometry
     /** return polygon from wkb */
     QgsPolygon asPolygon( unsigned char*& ptr, bool hasZValue );
 
+    static void checkRingIntersections( QList<Error> &errors,
+                                        int p0, int i0, const QgsPolyline &ring0,
+                                        int p1, int i1, const QgsPolyline &ring1 );
+
+    static bool geosRelOp( char( *op )( const GEOSGeometry*, const GEOSGeometry * ),
+                           QgsGeometry *a, QgsGeometry *b );
+
+
     static int refcount;
 }; // class QgsGeometry
 
