@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsmaplayerregistry.h 12276 2009-11-28 17:07:29Z jef $ */
+/* $Id: qgsmaplayerregistry.h 14264 2010-09-20 11:27:12Z mhugent $ */
 
 #ifndef QGSMAPLAYERREGISTRY_H
 #define QGSMAPLAYERREGISTRY_H
@@ -63,7 +63,7 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject
        Not emitting signal is useful when you want to use registry also for layers
        which won't be used in main map canvas but will be used in a special one
     */
-    QgsMapLayer *  addMapLayer( QgsMapLayer * theMapLayer, bool theEmitSignal = TRUE );
+    QgsMapLayer *addMapLayer( QgsMapLayer * theMapLayer, bool theEmitSignal = true );
 
     /** Remove a layer from qgis
        @note
@@ -73,7 +73,7 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject
        The layer being removed is deleted as well as the registry
        table entry.
     */
-    void removeMapLayer( QString theLayerId, bool theEmitSignal = TRUE );
+    void removeMapLayer( QString theLayerId, bool theEmitSignal = true );
 
     /** Remove all registered layers
        @note raises removedAll()
@@ -89,6 +89,11 @@ class CORE_EXPORT QgsMapLayerRegistry : public QObject
      * @note this method was added in QGIS 1.4
      */
     void clearAllLayerCaches();
+
+    /**Reload all provider data caches (currently used for WFS and WMS providers)
+      @note: this method was added in QGIS 1.6*/
+    void reloadAllLayers();
+
   signals:
 
     /** emitted when a layer is removed from the registry
