@@ -12,7 +12,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id$ */
+/* $Id: qgsmaptool.h 13187 2010-03-28 22:14:44Z jef $ */
 
 #ifndef QGSMAPTOOL_H
 #define QGSMAPTOOL_H
@@ -59,6 +59,10 @@ class GUI_EXPORT QgsMapTool : public QObject
     //! Key event for overriding. Default implementation does nothing.
     virtual void keyPressEvent( QKeyEvent* e );
 
+    //! Key event for overriding. Default implementation does nothing.
+    //! Added in version 1.1
+    virtual void keyReleaseEvent( QKeyEvent* e );
+
     //! Called when rendering has finished. Default implementation does nothing.
     virtual void renderComplete();
 
