@@ -14,15 +14,46 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgspoint.h 11825 2009-10-21 14:59:44Z timlinux $ */
+/* $Id: qgspoint.h 13005 2010-03-05 12:28:08Z mhugent $ */
 
 #ifndef QGSPOINT_H
 #define QGSPOINT_H
 
 #include <iostream>
-
 #include <QString>
 
+/** \ingroup core
+ * A class to represent a vector.
+ * Currently no Z axis / 2.5D support is implemented.
+ */
+
+class CORE_EXPORT QgsVector
+{
+    double m_x, m_y;
+
+  public:
+    QgsVector();
+    QgsVector( double x, double y );
+
+    QgsVector operator-( void ) const;
+    QgsVector operator*( double scalar ) const;
+    QgsVector operator/( double scalar ) const;
+    double operator*( QgsVector v ) const;
+    double length() const;
+
+    double x() const;
+    double y() const;
+
+    // perpendicular vector (rotated 90 counter-clockwise)
+    QgsVector perpVector() const;
+
+    double angle( void ) const;
+    double angle( QgsVector v ) const;
+    QgsVector rotateBy( double rot ) const;
+    QgsVector normal() const;
+
+};
+
 /** \ingroup core
  * A class to represent a point geometry.
  * Currently no Z axis / 2.5D support is implemented.
@@ -90,7 +121,7 @@ class CORE_EXPORT QgsPoint
     //! String representation of the point (x,y)
     QString toString() const;
 
-    //! As above but with precision for string representaiton of a point
+    //! As above but with precision for string representation of a point
     QString toString( int thePrecision ) const;
 
     /** Return a string representation as degrees minutes seconds.
@@ -113,6 +144,10 @@ class CORE_EXPORT QgsPoint
     /**Returns the squared distance between this and other point*/
     double sqrDist( const QgsPoint& other ) const;
 
+    /**Returns the minimum distance between this point and a segment
+    @note added in QGIS 1.5*/
+    double sqrDistToSegment( double x1, double y1, double x2, double y2, QgsPoint& minDistPoint ) const;
+
     //! equality operator
     bool operator==( const QgsPoint &other );
 
@@ -131,6 +166,12 @@ class CORE_EXPORT QgsPoint
     //! 3 if point is on open ray b.
     int onSegment( const QgsPoint& a, const QgsPoint& b ) const;
 
+    QgsVector operator-( QgsPoint p ) const { return QgsVector( m_x - p.m_x, m_y - p.m_y ); }
+    QgsPoint &operator+=( const QgsVector &v ) { *this = *this + v; return *this; }
+    QgsPoint &operator-=( const QgsVector &v ) { *this = *this - v; return *this; }
+    QgsPoint operator+( const QgsVector &v ) const { return QgsPoint( m_x + v.x(), m_y + v.y() ); }
+    QgsPoint operator-( const QgsVector &v ) const { return QgsPoint( m_x - v.x(), m_y - v.y() ); }
+
   private:
 
     //! x coordinate
@@ -140,7 +181,7 @@ class CORE_EXPORT QgsPoint
     double m_y;
 
 
-}; // class QgsPOint
+}; // class QgsPoint
 
 
 inline bool operator==( const QgsPoint &p1, const QgsPoint &p2 )
@@ -158,6 +199,4 @@ inline std::ostream& operator << ( std::ostream& os, const QgsPoint &p )
   return os;
 }
 
-
 #endif //QGSPOINT_H
-
