@@ -17,7 +17,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsproject.h 12350 2009-12-07 17:14:41Z wonder $ */
+/* $Id: qgsproject.h 14319 2010-09-30 22:53:18Z jef $ */
 
 #ifndef QGSPROJECT_H
 #define QGSPROJECT_H
@@ -144,7 +144,7 @@ class CORE_EXPORT QgsProject : public QObject
         implementations fails.  Since the read()s are invoked from qgisapp,
         then qgisapp handles the exception.  It prompts the user for the new
         location of the data, if any.  If there is a new location, the Dom
-        node associated with the layer has its <datasource> tag corrected.
+        node associated with the layer has its datasource tag corrected.
         Then that node is passed to this member function to be re-opened.
 
      */
@@ -230,7 +230,7 @@ class CORE_EXPORT QgsProject : public QObject
     QString readEntry( QString const & scope, const QString & key, const QString & def = QString::null, bool * ok = 0 ) const;
     int readNumEntry( QString const & scope, const QString & key, int def = 0, bool * ok = 0 ) const;
     double readDoubleEntry( QString const & scope, const QString & key, double def = 0, bool * ok = 0 ) const;
-    bool readBoolEntry( QString const & scope, const QString & key, bool def = FALSE, bool * ok = 0 ) const;
+    bool readBoolEntry( QString const & scope, const QString & key, bool def = false, bool * ok = 0 ) const;
     //@}
 
 
