@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsrasterdataprovider.h 12276 2009-11-28 17:07:29Z jef $ */
+/* $Id: qgsrasterdataprovider.h 14319 2010-09-30 22:53:18Z jef $ */
 
 /* Thank you to Marco Hugentobler for the original vector DataProvider */
 
@@ -115,10 +115,13 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider
      */
     virtual QString metadata() = 0;
 
+    /** \brief Identify raster value(s) found on the point position */
+    virtual bool identify( const QgsPoint & point, QMap<QString, QString>& results );
+
     /**
      * \brief Identify details from a server (e.g. WMS) from the last screen update
      *
-     * \param point[in]  The pixel coordinate (as it was displayed locally on screen)
+     * \param[in] point  The pixel coordinate (as it was displayed locally on screen)
      *
      * \return  A text document containing the return from the WMS server
      *
@@ -130,6 +133,23 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider
      */
     virtual QString identifyAsText( const QgsPoint& point ) = 0;
 
+    /**
+     * \brief Identify details from a server (e.g. WMS) from the last screen update
+     *
+     * \param[in] point  The pixel coordinate (as it was displayed locally on screen)
+     *
+     * \return  A html document containing the return from the WMS server
+     *
+     * \note WMS Servers prefer to receive coordinates in image space, therefore
+     *       this function expects coordinates in that format.
+     *
+     * \note  The arbitraryness of the returned document is enforced by WMS standards
+     *        up to at least v1.3.0
+     *
+     * \note  added in 1.5
+     */
+    virtual QString identifyAsHtml( const QgsPoint& point ) = 0;
+
     /**
      * \brief   Returns the caption error text for the last error in this provider
      *
@@ -152,6 +172,14 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider
      */
     virtual QString lastError() = 0;
 
+    /**
+     * \brief   Returns the format of the error text for the last error in this provider
+     *
+     * \note added in 1.6
+     */
+    virtual QString lastErrorFormat();
+
+
     /**Returns the dpi of the output device.
       @note: this method was added in version 1.2*/
     int dpi() const {return mDpi;}
