@@ -6,7 +6,7 @@
  email                : tim@linfiniti.com
 ***************************************************************************/
 /*
- * Peter J. Ersts - contirbuted to the refactoring and maintenance of this class
+ * Peter J. Ersts - contributed to the refactoring and maintenance of this class
  * B. Morley - added functions to convert this class to a data provider interface
  * Frank Warmerdam - contributed bug fixes and migrated class to use all GDAL_C_API calls
  */
@@ -18,7 +18,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsrasterlayer.h 12600 2009-12-23 16:14:24Z jef $ */
+/* $Id: qgsrasterlayer.h 14436 2010-10-26 09:09:51Z jef $ */
 #ifndef QGSRASTERLAYER_H
 #define QGSRASTERLAYER_H
 
@@ -59,7 +59,6 @@ class QgsMapToPixel;
 class QgsRectangle;
 class QgsRasterBandStats;
 class QgsRasterPyramid;
-class QgsRasterLayerProperties;
 class QImage;
 class QPixmap;
 class QSlider;
@@ -83,7 +82,7 @@ class QLibrary;
  *  PalettedSingleBandGray -> a PaletteD layer drawn in gray scale (using only one of the color components)
  *  PalettedSingleBandPseudoColor -> a PaletteD layer having only one of its color components rendered as psuedo color
  *  PalettedMultiBandColor -> a PaletteD image where the bands contains 24bit color info and 8 bits is pulled out per color
- *  MultiBandSingleGandGray -> a layer containing 2 or more bands, but using only one band to produce a grayscale image
+ *  MultiBandSingleBandGray -> a layer containing 2 or more bands, but using only one band to produce a grayscale image
  *  MultiBandSingleBandPseudoColor -> a layer containing 2 or more bands, but using only one band to produce a pseudocolor image
  *  MultiBandColor -> a layer containing 2 or more bands, mapped to the three RGBcolors. In the case of a multiband with only two bands, one band will have to be mapped to more than one color
  *
@@ -239,6 +238,8 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
       PalettedSingleBandPseudoColor, // a "Palette" layerdrawn using a pseudocolor algorithm
       PalettedMultiBandColor,         // currently not supported
       MultiBandSingleGandGray,        // a layer containing 2 or more bands, but a single band drawn as a range of gray colors
+      //added in 1.6 to fix naming glitch
+      MultiBandSingleBandGray = MultiBandSingleGandGray, // a layer containing 2 or more bands, but a single band drawn as a range of gray colors
       MultiBandSingleBandPseudoColor, //a layer containing 2 or more bands, but a single band drawn using a pseudocolor algorithm
       MultiBandColor                  //a layer containing 2 or more bands, mapped to RGB color space.
       //In the case of a multiband with only two bands, one band will be mapped to more than one color.
@@ -284,8 +285,9 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      *  returned in retError.
      */
     static bool isValidRasterFileName( const QString & theFileNameQString, QString &retError );
-
     static bool isValidRasterFileName( const QString & theFileNameQString );
+    static QStringList subLayers( GDALDatasetH dataset );
+
 
     /** Return time stamp for given file name */
     static QDateTime lastModified( const QString &  name );
@@ -440,12 +442,20 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Wrapper for GDALComputeRasterMinMax with the estimate option */
     void computeMinimumMaximumEstimates( QString theBand, double* theMinMax );
 
+    /** \brief Wrapper for GDALComputeRasterMinMax with the estimate option
+      \note added in v1.6 */
+    void computeMinimumMaximumEstimates( int theBand, double& theMin, double& theMax );
+
     /** \brief Compute the actual minimum maximum pixel values based on the current (last) display extent */
     void computeMinimumMaximumFromLastExtent( int theBand, double* theMinMax );
 
     /** \brief Compute the actual minimum maximum pixel values based on the current (last) display extent */
     void computeMinimumMaximumFromLastExtent( QString theBand, double* theMinMax );
 
+    /**  \brief Compute the actual minimum maximum pixel values based on the current (last) display extent
+      \note added in v1.6 */
+    void computeMinimumMaximumFromLastExtent( int theBand, double& theMin, double& theMax );
+
     /** \brief Get a pointer to the contrast enhancement for the selected band */
     QgsContrastEnhancement* contrastEnhancement( unsigned int theBand );
 
@@ -461,6 +471,10 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** Returns the data provider in a const-correct manner */
     const QgsRasterDataProvider* dataProvider() const;
 
+    /**Synchronises with changes in the datasource
+    @note added in version 1.6*/
+    virtual void reload();
+
     /** \brief This is called when the view on the raster layer needs to be redrawn */
     bool draw( QgsRenderContext& rendererContext );
 
@@ -488,6 +502,11 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Identify arbitrary details from the WMS server found on the point position */
     QString identifyAsText( const QgsPoint & point );
 
+    /** \brief Identify arbitrary details from the WMS server found on the point position
+     * @note added in 1.5
+     */
+    QString identifyAsHtml( const QgsPoint & point );
+
     /** \brief Currently returns always false */
     bool isEditable() const;
 
@@ -570,6 +589,9 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Mutator for setting the maximum value for contrast enhancement */
     void setMaximumValue( QString theBand, double theValue, bool theGenerateLookupTableFlag = true );
 
+    /** \brief Sets the minimum and maximum values for the band(s) currently being displayed using the only pixel values from the last/current extent */
+    void setMinimumMaximumUsingLastExtent();
+
     /** \brief Mutator for setting the minimum value for contrast enhancement */
     void setMinimumValue( unsigned int theBand, double theValue, bool theGenerateLookupTableFlag = true );
 
@@ -579,7 +601,8 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Mutator that allows the  NO_DATA entry for this raster to be overridden */
     void setNoDataValue( double theNoData );
 
-    /** \brief Set the raster shader function to a user defined function */
+    /** \brief Set the raster shader function to a user defined function
+      \note ownership of the shader function is transfered to raster shader */
     void setRasterShaderFunction( QgsRasterShaderFunction* theFunction );
 
     /** \brief Mutator for red band name (allows alternate mappings e.g. map blue as red color) */
@@ -596,6 +619,10 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
 
     /** \brief Draws a thumbnail of the rasterlayer into the supplied pixmap pointer */
     void thumbnailAsPixmap( QPixmap * theQPixmap );
+    /** \brief Draws a thumbnail of the rasterlayer into the supplied QImage pointer
+     * @note added in QGIS 1.6
+     * */
+    void thumbnailAsImage( QImage * thepImage );
 
     /** \brief Emit a signal asking for a repaint. (inherited from maplayer) */
     void triggerRepaint();
@@ -881,4 +908,59 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     bool mValidNoDataValue;
 };
 
+/*#include <QColor>
+
+typedef void* GDALRasterBandH;
+class QgsMapToPixel;
+struct QgsRasterViewPort;
+class QImage;
+class QPainter;*/
+
+/**A class encapsulates reading from a raster band and drawing the pixels to a painter.
+   The class allows sequential reading of the scan lines and setting the image scan line pixels. It automatically decides
+   on how much of the band / image should stay in virtual memory at a time*/
+class CORE_EXPORT QgsRasterImageBuffer
+{
+  public:
+    QgsRasterImageBuffer( GDALRasterBandH rasterBand, QPainter* p,
+                          QgsRasterViewPort* viewPort, const QgsMapToPixel* mapToPixel, double* mGeoTransform );
+    ~QgsRasterImageBuffer();
+    void reset( int maxPixelsInVirtualMemory = 5000000 );
+    /**Returns a pointer to the next scan line (or 0 if end)*/
+    bool nextScanLine( QRgb** imageScanLine, void** rasterScanLine );
+
+    void setWritingEnabled( bool enabled ) { mWritingEnabled = enabled; }
+
+  private:
+    QgsRasterImageBuffer(); //forbidden
+    /**Creates next part image. Returns false if at end*/
+    bool createNextPartImage();
+
+    /**Peter's fix for zoomed in rasters*/
+    void drawPixelRectangle();
+
+    GDALRasterBandH mRasterBand; //raster band
+    QPainter* mPainter;
+    QgsRasterViewPort* mViewPort;
+    const QgsMapToPixel* mMapToPixel;
+    double* mGeoTransform;
+
+    bool mValid;
+    /**True (default), if values are written to an image. If false, the class only reads the values, but does not create an image*/
+    bool mWritingEnabled;
+    /**Draws the raster pixels as rectangles. This is only used if the map units per pixel is very, very small*/
+    bool mDrawPixelRect;
+    int mCurrentRow;
+    int mNumPartImages; //number of part images
+    int mNumRasterRowsPerPart; //number of (raster source) rows per part
+    int mCurrentPartRasterMin; //minimum (raster source) row of current image
+    int mCurrentPartRasterMax; //maximum (raster source) row of current image
+    int mCurrentPartImageRow; //current image row
+    int mNumCurrentImageRows; //number of image rows for the current part
+
+    //current memory image and gdal scan data
+    QImage* mCurrentImage;
+    void* mCurrentGDALData;
+};
+
 #endif
