@@ -12,17 +12,17 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsrubberband.h 12528 2009-12-20 12:29:07Z jef $ */
+/* $Id: qgsrubberband.h 14319 2010-09-30 22:53:18Z jef $ */
 #ifndef QGSRUBBERBAND_H
 #define QGSRUBBERBAND_H
 
 #include "qgsmapcanvasitem.h"
+#include "qgsgeometry.h"
 #include <QBrush>
 #include <QList>
 #include <QPen>
 #include <QPolygon>
 
-class QgsGeometry;
 class QgsVectorLayer;
 class QPaintEvent;
 
@@ -58,18 +58,35 @@ class GUI_EXPORT QgsRubberBand: public QgsMapCanvasItem
     @param geom the geometry object
     @param layer the layer containing the feature, used for coord transformation to map
     crs. In case of 0 pointer, the coordinates are not going to be transformed.
-    @param render the maprender object (used for coord transformation)*/
+    */
     void setToGeometry( QgsGeometry* geom, QgsVectorLayer* layer );
 
+    /**Add the geometry of an existing feature to a rubberband
+     This is useful for multi feature highlighting.
+    @param geom the geometry object
+    @param layer the layer containing the feature, used for coord transformation to map
+    crs. In case of 0 pointer, the coordinates are not going to be transformed.
+    @note added in 1.5
+    */
+    void addGeometry( QgsGeometry* geom, QgsVectorLayer* layer );
+
     /**Adds translation to original coordinates (all in map coordinates)*/
     void setTranslationOffset( double dx, double dy );
 
+    /**Returns number of geometries
+     * added in 1.5 */
+    int size() const;
+
     /**Returns count of vertices in all lists of mPoint*/
     int numberOfVertices() const;
 
     /**Return vertex*/
     const QgsPoint *getPoint( int i, int j = 0 ) const;
 
+    /**Returns the rubberband as a Geometry.
+    * added in 1.6 */
+    QgsGeometry* asGeometry();
+
   protected:
     virtual void paint( QPainter* p );
 
@@ -87,6 +104,9 @@ class GUI_EXPORT QgsRubberBand: public QgsMapCanvasItem
     double mTranslationOffsetY;
 
     QgsRubberBand();
+
+    static QgsPolyline getPolyline( const QList<QgsPoint> & points );
+
 };
 
 #endif
