@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsvectorlayer.h 12537 2009-12-20 15:39:05Z wonder $ */
+/* $Id: qgsvectorlayer.h 14455 2010-10-30 14:27:23Z jef $ */
 
 #ifndef QGSVECTORLAYER_H
 #define QGSVECTORLAYER_H
@@ -44,6 +44,7 @@ class QgsRenderer;
 class QgsUndoCommand;
 class QgsVectorDataProvider;
 class QgsVectorOverlay;
+class QgsSingleSymbolRendererV2;
 
 class QgsRectangle;
 
@@ -74,9 +75,11 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
       CheckBox,    /* added in 1.4 */
       FileName,
       Enumeration,
-      Immutable,   /*The attribute value should not be changed in the attribute form*/
-      Hidden,      /*The attribute value should not be shown in the attribute form @added in 1.4 */
-      TextEdit     /*multiline edit @added in 1.4*/
+      Immutable,   /* The attribute value should not be changed in the attribute form*/
+      Hidden,      /* The attribute value should not be shown in the attribute form @added in 1.4 */
+      TextEdit,    /* multiline edit @added in 1.4*/
+      Calendar,    /* calendar widget @added in 1.5 */
+      DialRange,   /* dial range @added in 1.5 */
     };
 
     struct RangeData
@@ -91,8 +94,8 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     };
 
     /** Constructor */
-    QgsVectorLayer( QString path = 0, QString baseName = 0,
-                    QString providerLib = 0, bool loadDefaultStyleFlag = true );
+    QgsVectorLayer( QString path = QString::null, QString baseName = QString::null,
+                    QString providerLib = QString::null, bool loadDefaultStyleFlag = true );
 
     /** Destructor */
     virtual ~QgsVectorLayer();
@@ -107,7 +110,7 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     QString dataComment() const;
 
     /** Set the primary display field to be used in the identify results dialog */
-    void setDisplayField( QString fldName = 0 );
+    void setDisplayField( QString fldName = "" );
 
     /** Returns the primary display field name used in the identify results dialog */
     const QString displayField() const;
@@ -167,18 +170,30 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     /** Sets the renderer. If a renderer is already present, it is deleted */
     void setRenderer( QgsRenderer * r );
 
-    /** Return renderer V2. Added in QGIS 1.4 */
+    /** Return renderer V2.
+     * @note added in 1.4 */
     QgsFeatureRendererV2* rendererV2();
-    /** Set renderer V2. Added in QGIS 1.4 */
+    /** Set renderer V2.
+     * @note ddded in 1.4
+     */
     void setRendererV2( QgsFeatureRendererV2* r );
-    /** Return whether using renderer V2. Added in QGIS 1.4 */
+    /** Return whether using renderer V2.
+     * @note added in 1.4
+     */
     bool isUsingRendererV2();
-    /** set whether to use renderer V2 for drawing. Added in QGIS 1.4 */
+    /** set whether to use renderer V2 for drawing.
+     * @note added in 1.4
+     */
     void setUsingRendererV2( bool usingRendererV2 );
 
-    /** Draw layer with renderer V2. Added in QGIS 1.4 */
+    /** Draw layer with renderer V2.
+     * @note added in 1.4
+     */
     void drawRendererV2( QgsRenderContext& rendererContext, bool labeling );
-    /** Draw layer with renderer V2 using symbol levels. Added in QGIS 1.4 */
+
+    /** Draw layer with renderer V2 using symbol levels.
+     * @note added in 1.4
+     */
     void drawRendererV2Levels( QgsRenderContext& rendererContext, bool labeling );
 
     /** Returns point, line or polygon */
@@ -201,19 +216,19 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     virtual bool writeXml( QDomNode & layer_node, QDomDocument & doc );
 
     /** Read the symbology for the current layer from the Dom node supplied.
-    * @param QDomNode node that will contain the symbology definition for this layer.
+    * @param node node that will contain the symbology definition for this layer.
     * @param errorMessage reference to string that will be updated with any error messages
     * @return true in case of success.
     */
     bool readSymbology( const QDomNode& node, QString& errorMessage );
 
     /** Write the symbology for the layer into the docment provided.
-     *  @param QDomNode the node that will have the style element added to it.
-     *  @param QDomDocument the document that will have the QDomNode added.
-     * @param errorMessage reference to string that will be updated with any error messages
+     *  @param node the node that will have the style element added to it.
+     *  @param doc the document that will have the QDomNode added.
+     *  @param errorMessage reference to string that will be updated with any error messages
      *  @return true in case of success.
      */
-    bool writeSymbology( QDomNode&, QDomDocument& doc, QString& errorMessage ) const;
+    bool writeSymbology( QDomNode& node, QDomDocument& doc, QString& errorMessage ) const;
 
 
     /**
@@ -225,8 +240,8 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
      */
     virtual long featureCount() const;
 
-    /** Update the feature count
-     * @return long containing the number of features in the datasource
+    /** This function does nothing useful, it's kept only for compatibility.
+     * @todo to be removed
      */
     virtual long updateFeatureCount() const;
 
@@ -245,11 +260,23 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
      */
     virtual QString subsetString();
 
+    /**
+     * Select features with or without attributes in a given window.
+     * @param fetchAttributes indizes of attributes to fetch
+     * @param rect window (QgsRectangle() for all)
+     * @param fetchGeometry fetch features with geometry
+     * @param useIntersect fetch only features that actually intersect the window (not just the bounding box)
+     */
     void select( QgsAttributeList fetchAttributes,
                  QgsRectangle rect = QgsRectangle(),
                  bool fetchGeometry = true,
                  bool useIntersect = false );
 
+    /**
+     * fetch a feature (after select)
+     * @param feature buffer to read the feature into
+     * @return true, if a feature was fetched, false, if there are no more features
+     */
     bool nextFeature( QgsFeature& feature );
 
     /**Gets the feature at the given feature id. Considers the changed, added, deleted and permanent features
@@ -257,10 +284,11 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     bool featureAtId( int featureId, QgsFeature &f, bool fetchGeometries = true, bool fetchAttributes = true );
 
     /** Adds a feature
-        @param lastFeatureInBatch  If True, will also go to the effort of e.g. updating the extents.
+        @param f feature to add
+        @param alsoUpdateExtent If True, will also go to the effort of e.g. updating the extents.
         @return                    True in case of success and False in case of error
      */
-    bool addFeature( QgsFeature& f, bool alsoUpdateExtent = TRUE );
+    bool addFeature( QgsFeature& f, bool alsoUpdateExtent = true );
 
 
     /** Insert a new vertex before the given vertex number,
@@ -354,6 +382,10 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     /** Returns true if the provider is in editing mode */
     virtual bool isEditable() const;
 
+    /** Returns true if the provider is in read-only mode
+     * @note added in 1.6 */
+    virtual bool isReadOnly() const;
+
     /** Returns true if the provider has been modified since the last commit */
     virtual bool isModified() const;
 
@@ -377,8 +409,12 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
                          QgsSnappingResult > & snappingResults,
                          QgsSnapper::SnappingType snap_to );
 
+    /**Synchronises with changes in the datasource
+      @note added in version 1.6*/
+    virtual void reload();
+
     /** Draws the layer
-     *  @return FALSE if an error occurred during drawing
+     *  @return false if an error occurred during drawing
      */
     bool draw( QgsRenderContext& rendererContext );
 
@@ -394,8 +430,14 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     /** returns feature count after commit */
     int pendingFeatureCount();
 
+    /** Make layer read-only (editing disabled) or not
+     *  @return false if the layer is in editing yet
+     *  @note added in 1.6
+     */
+    bool setReadOnly( bool readonly = true );
+
     /** Sets whether some features are modified or not */
-    void setModified( bool modified = TRUE, bool onlyGeometryWasModified = FALSE );
+    void setModified( bool modified = true, bool onlyGeometryWasModified = false );
 
     /** Make layer editable */
     bool startEditing();
@@ -433,7 +475,7 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     bool deleteAttribute( int attr );
 
     /** Insert a copy of the given features into the layer  (but does not commit it) */
-    bool addFeatures( QgsFeatureList features, bool makeSelected = TRUE );
+    bool addFeatures( QgsFeatureList features, bool makeSelected = true );
 
     /** delete a feature from the layer (but does not commit it) */
     bool deleteFeature( int fid );
@@ -477,6 +519,12 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     /** set edit form (added in 1.4) */
     void setEditForm( QString ui );
 
+    /** get annotation form (added in 1.5)*/
+    QString annotationForm() const { return mAnnotationForm; }
+
+    /** set annotation form for layer (added in 1.5)*/
+    void setAnnotationForm( const QString& ui );
+
     /** get python function for edit form initialization (added in 1.4) */
     QString editFormInit();
 
@@ -548,13 +596,13 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
 
   public slots:
     /** Select feature by its ID, optionally emit signal selectionChanged() */
-    void select( int featureId, bool emitSignal = TRUE );
+    void select( int featureId, bool emitSignal = true );
 
     /** Deselect feature by its ID, optionally emit signal selectionChanged() */
-    void deselect( int featureId, bool emitSignal = TRUE );
+    void deselect( int featureId, bool emitSignal = true );
 
     /** Clear selection */
-    void removeSelection( bool emitSignal = TRUE );
+    void removeSelection( bool emitSignal = true );
 
     void triggerRepaint();
 
@@ -580,6 +628,14 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
 
     void attributeValueChanged( int fid, int idx, const QVariant & );
 
+    /** Signals emitted after committing changes */
+    void committedAttributesDeleted( const QString& layerId, const QgsAttributeIds& deletedAttributeIds );
+    void committedAttributesAdded( const QString& layerId, const QList<QgsField>& addedAttributes );
+    void committedFeaturesAdded( const QString& layerId, const QgsFeatureList& addedFeatures );
+    void committedFeaturesRemoved( const QString& layerId, const QgsFeatureIds& deletedFeatureIds );
+    void committedAttributeValuesChanges( const QString& layerId, const QgsChangedAttributesMap& changedAttributesValues );
+    void committedGeometriesChanges( const QString& layerId, const QgsGeometryMap& changedGeometries );
+
   private:                       // Private methods
 
     /** vector layers are not copyable */
@@ -624,9 +680,10 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
 
     /**Snaps to a geometry and adds the result to the multimap if it is within the snapping result
      @param startPoint start point of the snap
+     @param featureId id of feature
      @param geom geometry to snap
      @param sqrSnappingTolerance squared search tolerance of the snap
-     @param snappingResult list to which the result is appended
+     @param snappingResults list to which the result is appended
      @param snap_to snap to vertex or to segment
     */
     void snapToGeometry( const QgsPoint& startPoint, int featureId, QgsGeometry* geom, double sqrSnappingTolerance,
@@ -643,7 +700,7 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     static int currentVertexMarkerSize();
 
     /**Update feature with uncommited attribute updates*/
-    void updateFeatureAttributes( QgsFeature &f );
+    void updateFeatureAttributes( QgsFeature &f, bool all = false );
 
     /**Update feature with uncommited geometry updates*/
     void updateFeatureGeometry( QgsFeature &f );
@@ -660,6 +717,8 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     /** Record changed attribute, store in active command (if any) */
     void editAttributeChange( int featureId, int field, QVariant value );
 
+    /** Stop version 2 renderer and selected renderer (if required) */
+    void stopRendererV2( QgsRenderContext& rendererContext, QgsSingleSymbolRendererV2* selRenderer );
 
   private:                       // Private attributes
 
@@ -683,6 +742,9 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     /** Flag indicating whether the layer is in editing mode or not */
     bool mEditable;
 
+    /** Flag indicating whether the layer is in read-only mode (editing disabled) or not */
+    bool mReadOnly;
+
     /** Flag indicating whether the layer has been modified since the last commit */
     bool mModified;
 
@@ -770,10 +832,13 @@ class CORE_EXPORT QgsVectorLayer : public QgsMapLayer
     QMap< QString, QPair<QString, QString> > mCheckedStates;
 
     QString mEditForm, mEditFormInit;
+    //annotation form for this layer
+    QString mAnnotationForm;
 
     bool mFetching;
     QgsRectangle mFetchRect;
     QgsAttributeList mFetchAttributes;
+    QgsAttributeList mFetchProvAttributes;
     bool mFetchGeometry;
 
     QSet<int> mFetchConsidered;
