@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from qgis.core import *
@@ -15,23 +16,24 @@ class VisualDialog( QDialog, Ui_Dialog ):
       QObject.connect( self.inShape, SIGNAL( "currentIndexChanged(QString)" ), self.update )
     self.manageGui()
     self.cancel_close = self.buttonBox_2.button( QDialogButtonBox.Close )
+    self.buttonOk = self.buttonBox_2.button( QDialogButtonBox.Ok )
     self.progressBar.setValue( 0 )
     self.partProgressBar.setValue( 0 )
-    self.partProgressBar.setEnabled( False )
+    self.partProgressBar.setVisible( False )
     
   def keyPressEvent( self, e ):
     '''
     Reimplemented key press event:
     '''
     if ( e.modifiers() == Qt.ControlModifier or e.modifiers() == Qt.MetaModifier ) and e.key() == Qt.Key_C:
-      selection = self.lstUnique.selectedItems()
+      #selection = self.tblUnique.selectedItems()
       items = QString()
       if self.myFunction in ( 1, 2 ):
-	for rec in range( self.tblUnique.rowCount() ):
-	  items.append( self.tblUnique.item( rec, 0 ).text() + "\n" )
+        for rec in range( self.tblUnique.rowCount() ):
+          items.append( self.tblUnique.item( rec, 0 ).text() + "\n" )
       else:
-	for rec in range( self.tblUnique.rowCount() ):
-	  items.append( self.tblUnique.item( rec, 0 ).text() + ":" + self.tblUnique.item( rec, 1 ).text() + "\n" )
+        for rec in range( self.tblUnique.rowCount() ):
+          items.append( self.tblUnique.item( rec, 0 ).text() + ":" + self.tblUnique.item( rec, 1 ).text() + "\n" )
       if not items.isEmpty():
         clip_board = QApplication.clipboard()
         clip_board.setText( items )
@@ -56,9 +58,9 @@ class VisualDialog( QDialog, Ui_Dialog ):
         
   def accept( self ):
     if self.inShape.currentText() == "":
-      QMessageBox.information( self, "Error!", self.tr( "Please specify input vector layer" ) )
+      QMessageBox.information( self, self.tr("Error!"), self.tr( "Please specify input vector layer" ) )
     elif self.cmbField.isVisible() and self.cmbField.currentText() == "":
-      QMessageBox.information( self, "Error!", self.tr( "Please specify input field" ) )
+      QMessageBox.information( self, self.tr("Error!"), self.tr( "Please specify input field" ) )
     else:
       self.visual( self.inShape.currentText(), self.cmbField.currentText(), self.useSelected.checkState() )
   
@@ -109,42 +111,49 @@ class VisualDialog( QDialog, Ui_Dialog ):
     self.tblUnique.clearContents()
     self.tblUnique.setRowCount( 0 )
     self.lstCount.clear()
+    self.buttonOk.setEnabled( False )
+
     self.testThread = visualThread( self.iface.mainWindow(), self, self.myFunction, vlayer, myField, mySelection )
     QObject.connect( self.testThread, SIGNAL( "runFinished(PyQt_PyObject)" ), self.runFinishedFromThread )
     QObject.connect( self.testThread, SIGNAL( "runStatus(PyQt_PyObject)" ), self.runStatusFromThread )
     QObject.connect( self.testThread, SIGNAL( "runRange(PyQt_PyObject)" ), self.runRangeFromThread )
     QObject.connect( self.testThread, SIGNAL( "runPartRange(PyQt_PyObject)" ), self.runPartRangeFromThread )
     QObject.connect( self.testThread, SIGNAL( "runPartStatus(PyQt_PyObject)" ), self.runPartStatusFromThread )
-    self.cancel_close.setText( "Cancel" )
+    self.cancel_close.setText( self.tr("Cancel") )
     QObject.connect( self.cancel_close, SIGNAL( "clicked()" ), self.cancelThread )
+
+    QApplication.setOverrideCursor( Qt.WaitCursor )
     self.testThread.start()
     return True
 
   def cancelThread( self ):
     self.testThread.stop()
+    QApplication.restoreOverrideCursor()
+    self.buttonOk.setEnabled( True )
     
   def runFinishedFromThread( self, output ):
     self.testThread.stop()
-    
+    QApplication.restoreOverrideCursor()
+    self.buttonOk.setEnabled( True )
     result = output[ 0 ]
     numRows = len( result )
     self.tblUnique.setRowCount( numRows )
     if self.myFunction in ( 1, 2 ):
       self.tblUnique.setColumnCount( 1 )
       for rec in range( numRows ):
-	item = QTableWidgetItem( result[ rec ] )
-	self.tblUnique.setItem( rec, 0, item )
+        item = QTableWidgetItem( result[ rec ] )
+        self.tblUnique.setItem( rec, 0, item )
     else:
       self.tblUnique.setColumnCount( 2 )
       for rec in range( numRows ):
-	tmp = result[ rec ].split( ":" )
-	item = QTableWidgetItem( tmp[ 0 ] )
-	self.tblUnique.setItem( rec, 0, item )
-	item = QTableWidgetItem( tmp[ 1 ] )
-	self.tblUnique.setItem( rec, 1, item )
-	self.tblUnique.setHorizontalHeaderLabels( [ "Parameter", "Value" ] )
-	self.tblUnique.horizontalHeader().setResizeMode( 1, QHeaderView.ResizeToContents )
-	self.tblUnique.horizontalHeader().show()
+        tmp = result[ rec ].split( ":" )
+        item = QTableWidgetItem( tmp[ 0 ] )
+        self.tblUnique.setItem( rec, 0, item )
+        item = QTableWidgetItem( tmp[ 1 ] )
+        self.tblUnique.setItem( rec, 1, item )
+      self.tblUnique.setHorizontalHeaderLabels( [ self.tr("Parameter"), self.tr("Value") ] )
+      self.tblUnique.horizontalHeader().setResizeMode( 1, QHeaderView.ResizeToContents )
+      self.tblUnique.horizontalHeader().show()
     self.tblUnique.horizontalHeader().setResizeMode( 0, QHeaderView.Stretch )
     self.tblUnique.resizeRowsToContents()
     
@@ -162,11 +171,11 @@ class VisualDialog( QDialog, Ui_Dialog ):
   def runPartStatusFromThread( self, status ):
     self.partProgressBar.setValue( status )
     if status >= self.part_max:
-      self.partProgressBar.setEnabled( False )
+      self.partProgressBar.setVisible( False )
         
   def runPartRangeFromThread( self, range_vals ):
     self.part_max = range_vals[ 1 ]
-    self.partProgressBar.setEnabled( True )
+    self.partProgressBar.setVisible( True )
     self.partProgressBar.setRange( range_vals[ 0 ], range_vals[ 1 ] )
     
 class visualThread( QThread ):
@@ -195,7 +204,7 @@ class visualThread( QThread ):
     self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
 
   def stop(self):
-    self.running = False        
+    self.running = False
 
   def list_unique_values( self, vlayer, myField ):
     vprovider = vlayer.dataProvider()
@@ -207,8 +216,9 @@ class visualThread( QThread ):
     lstUnique = []
     nFeat = len( unique )
     nElement = 0
-    self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-    self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+    if nFeat > 0:
+      self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
+      self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
     for item in unique:
       nElement += 1
       self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
@@ -230,14 +240,16 @@ class visualThread( QThread ):
     first = True
     nElement = 0
     # determine selected field type
-    if ftools_utils.getFieldType( vlayer, myField ) == 'String':
+    if ftools_utils.getFieldType( vlayer, myField ) in (
+    'String', 'varchar', 'char', 'text'):
       fillVal = 0
       emptyVal = 0
       if self.mySelection: # only selected features
         selection = vlayer.selectedFeatures()
         nFeat = vlayer.selectedFeatureCount()
-        self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-        self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+        if nFeat > 0:
+          self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
+          self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
         for f in selection:
           atMap = f.attributeMap()
           lenVal = float( len( atMap[ index ].toString() ) )
@@ -258,8 +270,10 @@ class visualThread( QThread ):
           self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
       else: # there is no selection, process the whole layer
         nFeat = vprovider.featureCount()
-        self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-        self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+	if nFeat > 0:
+          self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
+          self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+        vprovider.select( allAttrs )
         while vprovider.nextFeature( feat ):
           atMap = feat.attributeMap()
           lenVal = float( len( atMap[ index ].toString() ) )
@@ -279,27 +293,32 @@ class visualThread( QThread ):
           nElement += 1
           self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
       nVal= float( len( values ) )
-      if nVal > 0.00:
+      if nVal > 0:
         meanVal = sumVal / nVal
-      lstStats = []
-      lstStats.append( self.tr( "Max. len:" ) + unicode( maxVal ) )
-      lstStats.append( self.tr( "Min. len:" ) + unicode( minVal ) )
-      lstStats.append( self.tr( "Mean. len:" ) + unicode( meanVal ) )
-      lstStats.append( self.tr( "Filled:" ) + unicode( fillVal ) )
-      lstStats.append( self.tr( "Empty:" ) + unicode( emptyVal ) )
-      lstStats.append( self.tr( "N:" ) + unicode( nVal ) )
-      return ( lstStats, [] )
+        lstStats = []
+        lstStats.append( self.tr( "Max. len:" ) + unicode( maxVal ) )
+        lstStats.append( self.tr( "Min. len:" ) + unicode( minVal ) )
+        lstStats.append( self.tr( "Mean. len:" ) + unicode( meanVal ) )
+        lstStats.append( self.tr( "Filled:" ) + unicode( fillVal ) )
+        lstStats.append( self.tr( "Empty:" ) + unicode( emptyVal ) )
+        lstStats.append( self.tr( "N:" ) + unicode( nVal ) )
+        return ( lstStats, [] )
+      else:
+        return ( ["Error:No features selected!"], [] )
     else: # numeric field
-      stdVal = 0
-      cvVal = 0
-      rangeVal = 0
-      medianVal = 0
+      stdVal = 0.00
+      cvVal = 0.00
+      rangeVal = 0.00
+      medianVal = 0.00
+      maxVal = 0.00
+      minVal = 0.00
       if self.mySelection: # only selected features
         selection = vlayer.selectedFeatures()
         nFeat = vlayer.selectedFeatureCount()
-	uniqueVal = ftools_utils.getUniqueValuesCount( vlayer, index, True )
-        self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-        self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+        uniqueVal = ftools_utils.getUniqueValuesCount( vlayer, index, True )
+        if nFeat > 0:
+          self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
+          self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
         for f in selection:
           atMap = f.attributeMap()
           value = float( atMap[ index ].toDouble()[ 0 ] )
@@ -316,9 +335,11 @@ class visualThread( QThread ):
           self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
       else: # there is no selection, process the whole layer
         nFeat = vprovider.featureCount()
-	uniqueVal = ftools_utils.getUniqueValuesCount( vlayer, index, False )
-        self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-        self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+        uniqueVal = ftools_utils.getUniqueValuesCount( vlayer, index, False )
+	if nFeat > 0:
+          self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
+          self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+        vprovider.select( allAttrs )
         while vprovider.nextFeature( feat ):
           atMap = feat.attributeMap()
           value = float( atMap[ index ].toDouble()[ 0 ] )
@@ -334,33 +355,35 @@ class visualThread( QThread ):
           nElement += 1
           self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
       nVal= float( len( values ) )
-      rangeVal = maxVal - minVal
       if nVal > 0.00:
+        rangeVal = maxVal - minVal
         meanVal = sumVal / nVal
         if meanVal != 0.00:
           for val in values:
             stdVal += ( ( val - meanVal ) * ( val - meanVal ) )
           stdVal = math.sqrt( stdVal / nVal )
           cvVal = stdVal / meanVal
-      if nVal > 1:
-	  lstVal = values
-	  lstVal.sort()
-	  if ( nVal % 2 ) == 0:
-	    medianVal = 0.5 * ( lstVal[ int( ( nVal - 1 ) / 2 ) ] + lstVal[ int( ( nVal ) / 2 ) ] )
-	  else:
-	    medianVal = lstVal[ int( ( nVal + 1 ) / 2 ) ]
-      lstStats = []
-      lstStats.append( self.tr( "Mean:" ) + unicode( meanVal ) )
-      lstStats.append( self.tr( "StdDev:" ) + unicode( stdVal ) )
-      lstStats.append( self.tr( "Sum:" ) + unicode( sumVal) )
-      lstStats.append( self.tr( "Min:" ) + unicode( minVal ) )
-      lstStats.append( self.tr( "Max:" ) + unicode( maxVal ) )
-      lstStats.append( self.tr( "N:" ) + unicode( nVal ) )
-      lstStats.append( self.tr( "CV:" ) + unicode( cvVal ) )
-      lstStats.append( self.tr( "Number of unique values:" ) + unicode( uniqueVal ) )
-      lstStats.append( self.tr( "Range:" ) + unicode( rangeVal ) )
-      lstStats.append( self.tr( "Median:" ) + unicode( medianVal ) )
-      return ( lstStats, [] )
+        if nVal > 1:
+            lstVal = values
+            lstVal.sort()
+            if ( nVal % 2 ) == 0:
+                medianVal = 0.5 * ( lstVal[ int( ( nVal - 1 ) / 2 ) ] + lstVal[ int( ( nVal ) / 2 ) ] )
+            else:
+                medianVal = lstVal[ int( ( nVal + 1 ) / 2 ) ]
+        lstStats = []
+        lstStats.append( self.tr( "Mean:" ) + unicode( meanVal ) )
+        lstStats.append( self.tr( "StdDev:" ) + unicode( stdVal ) )
+        lstStats.append( self.tr( "Sum:" ) + unicode( sumVal) )
+        lstStats.append( self.tr( "Min:" ) + unicode( minVal ) )
+        lstStats.append( self.tr( "Max:" ) + unicode( maxVal ) )
+        lstStats.append( self.tr( "N:" ) + unicode( nVal ) )
+        lstStats.append( self.tr( "CV:" ) + unicode( cvVal ) )
+        lstStats.append( self.tr( "Number of unique values:" ) + unicode( uniqueVal ) )
+        lstStats.append( self.tr( "Range:" ) + unicode( rangeVal ) )
+        lstStats.append( self.tr( "Median:" ) + unicode( medianVal ) )
+        return ( lstStats, [] )
+      else:
+        return ( ["Error:No features selected!"], [] )
 
   def nearest_neighbour_analysis( self, vlayer ):
     vprovider = vlayer.dataProvider()
@@ -376,8 +399,9 @@ class visualThread( QThread ):
     vprovider.rewind()
     nFeat = vprovider.featureCount()
     nElement = 0
-    self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-    self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+    if nFeat > 0:
+      self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
+      self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
     while vprovider.nextFeature( feat ):
       neighbourID = index.nearestNeighbor( feat.geometry().asPoint(), 2 )[ 1 ]
       vprovider.featureAtId( neighbourID, neighbour, True, [] )
@@ -409,12 +433,14 @@ class visualThread( QThread ):
     lstErrors = []
     nFeat = vprovider.featureCount()
     nElement = 0
-    self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-    self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+    if nFeat > 0:
+      self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
+      self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+
     while vprovider.nextFeature( feat ):
       geom = QgsGeometry( feat.geometry() )
-      nElement += 1
       self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
+      nElement += 1
       if geom.isMultipart():
         polygons = geom.asMultiPolygon()
         for polygon in polygons:
@@ -489,7 +515,8 @@ class visualThread( QThread ):
         for j in range( i+1, len(h)-1 ):
           if QgsGeometry().fromPolyline( [ h[ i ], h[ i + 1 ] ] ).intersects( QgsGeometry().fromPolyline( [ h[ j ], h[ j + 1 ] ] ) ):
             count += 1
-        if count > 2:
+
+        if (i==0 and count>2) or (i>0 and count>1):
           self.emit( SIGNAL( "runPartStatus(PyQt_PyObject)" ), cPart )
           return True
 
