@@ -255,7 +255,7 @@ class Qgis2Map:
       logmsg += "\n> We only saved the LAYER portion of the map file. \nMerge this into an excisting map file to see it working\n"
     else:
       logmsg += "\n> If this mapfile is accessible by your mapserver, you\nshould be able to see the capabilities by firing this url:\n" + self.mapServerUrl + "?MAP="+self.mapFile+"&SERVICE=WMS&VERSION=1.1.1&REQUEST=GetCapabilities\n"
-      logmsg += "\n> if this mapfile is accessible by your mapserver, you\nshould be able to see a map by firing this url:\n" + self.mapServerUrl + "?MAP="+self.mapFile+"&SERVICE=WMS&LAYERS=ALL&MODE=MAP\n"
+      logmsg += "\n> if this mapfile is accessible by your mapserver, you\nshould be able to see a map by firing this url:\n" + self.mapServerUrl + "?MAP="+self.mapFile+"&LAYERS=ALL&MODE=MAP\n"
     return logmsg
 
   # Write the general parts of the map section
@@ -284,7 +284,7 @@ class Qgis2Map:
     stringToAddTo = ""
 
     xmin = self.qgs.getElementsByTagName("xmin")
-    stringToAddTo += "    EXTENT "
+    stringToAddTo += "  EXTENT "
     stringToAddTo += xmin[0].childNodes[0].nodeValue.encode('utf-8')
     stringToAddTo += " "
 
@@ -382,8 +382,11 @@ class Qgis2Map:
     self.outFile.write("\n")
 
     destsrs = self.qgs.getElementsByTagName("destinationsrs")[0]
-    epsg = destsrs.getElementsByTagName("epsg")[0].childNodes[0].nodeValue.encode("utf-8")
-
+    try:
+      epsg = destsrs.getElementsByTagName("srid")[0].childNodes[0].nodeValue.encode("utf-8")
+    except:
+      # default to epsg
+      epsg="4326"
     self.outFile.write("    # WMS server settings\n")
     self.outFile.write("    METADATA\n")
     self.outFile.write("      'ows_title'           '" + self.mapName + "'\n")
@@ -418,31 +421,40 @@ class Qgis2Map:
   # compared to mapserver
   def writeMapLayers(self):
     resultMsg = ''
-    # get the list of legend nodes so the layers can be written in the proper order
-    legend_nodes = self.qgs.getElementsByTagName("legendlayer")
-    self.z_order = list()
-    for legend_node in legend_nodes:
-        self.z_order.append(legend_node.getAttribute("name").encode('utf-8').replace("\"", "").replace(" ","_"))
-
+    # get the layers from the legend to be able to determine the order later
+    legend_layers = self.qgs.getElementsByTagName("legendlayerfile")
+    self.layer_order = list()
+    for legend_layer in legend_layers:
+        self.layer_order.append(legend_layer.getAttribute("layerid").encode('utf-8'))
     # get the list of maplayer nodes
     maplayers = self.qgs.getElementsByTagName("maplayer")
     print "Processing ", len(maplayers), " layers"
     count = 0
     layer_list = dict()
+    layer_names = []
     for lyr in maplayers:
       count += 1
-      print "Processing layer ", count 
+      print "Processing layer ", count
       # The attributes of the maplayer tag contain the scale dependent settings,
       # visibility, and layer type
       layer_def = "  LAYER\n"
       # store name of the layer - replace space with underscore for wms compliance
       layer_name = lyr.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8').replace("\"", "").replace(" ","_")
+      # layername is not unique in qgis, store id of layer
+      layer_id = lyr.getElementsByTagName("id")[0].childNodes[0].nodeValue.encode('utf-8')
       # first check to see if there is a name
       if len(layer_name) > 0:
-        layer_def += "    NAME '%s'\n" % layer_name
+        # WMS layernames should be unique, so
+        # if the layer_name already excists in our layer_list of names:
+        if layer_name in layer_names:
+          # we give it the old name plus number
+          layer_name = layer_name + str(count)
       else:
-        # if no name for the layer, manufacture one  
-        layer_def += "    NAME 'LAYER%s'\n" % count
+        # if no name for the layer, manufacture one
+        layer_name = 'layer' + str(count)
+      # store the name to be able to check for double names
+      layer_names.append(layer_name)
+      layer_def += "    NAME '%s'\n" % layer_name
 
       if lyr.getAttribute("type").encode('utf-8') == 'vector':  
         layer_def += "    TYPE " + lyr.getAttribute("geometry").encode('utf-8').upper() + "\n"
@@ -489,8 +501,6 @@ class Qgis2Map:
         #layer_def += "    DATA '\"" + uri.geometryColumn() + "\" FROM " + uri.quotedTablename() + "'\n"
         #layer_def += "    DATA '" + uri.geometryColumn() + " FROM " + uri.quotedTablename() + "'\n"
         layer_id = lyr.getElementsByTagName("id")[0].childNodes[0].nodeValue.encode("utf-8")
-        # TODO: check if this project is actually loaded in QGis
-        # only in loaded project files it's possible to determine the primary key of a postgis table
         uniqueId = self.getPrimaryKey(layer_id, uri.table())
         # %tablename% is returned when no uniqueId is found: inform user
         if uniqueId.find("%") >= 0:
@@ -537,13 +547,16 @@ class Qgis2Map:
         layer_def += "      'wms_style' '" + ','.join(wmsStyles) + "'\n"
         layer_def += "    END\n"
 
+      elif providerString == 'ogr':
+        layer_def += "    DATA '" + dataString.split('|')[0] + "'\n"
+
       else: 
-        # its a standard ogr, gdal or grass layer
+        # it's a standard gdal or grass layer
         layer_def += "    DATA '" + dataString + "'\n"
       
       # WMS settings for all layers
       layer_def += "    METADATA\n"
-      layer_def += "      'ows_title' '" + lyr.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8').replace("\"", "") + "'\n"
+      layer_def += "      'ows_title' '" + layer_name + "'\n"
       layer_def += "    END\n"
 
       layer_def += "    STATUS OFF\n"
@@ -551,7 +564,6 @@ class Qgis2Map:
       # turn status in MapServer on or off based on visibility in QGis:
 #      layer_id = lyr.getElementsByTagName("id")[0].childNodes[0].nodeValue.encode("utf-8")
 #      legendLayerNode = self.legendlayerfileNodesById[layer_id]
-
 #      if legendLayerNode.getAttribute("visible").encode("utf-8") == "1":
 #        layer_def += "    STATUS ON\n"
 #      else:
@@ -586,18 +598,24 @@ class Qgis2Map:
           layer_def += "    MINSCALE " + minscale + "\n"
         if maxscale > '':
           layer_def += "    MAXSCALE " + maxscale + "\n"
-
-      
       # Check for label field (ie LABELITEM) and label status
       try:
-        labelOn    = lyr.getElementsByTagName(     "label")[0].childNodes[0].nodeValue.encode('utf-8')
-        labelField = lyr.getElementsByTagName("labelfield")[0].childNodes[0].nodeValue.encode('utf-8')
-        if labelField != '' and labelField is not None and labelOn == "1":
+        labelElements = lyr.getElementsByTagName("label")
+        labelOn = '0'
+        labelField = None
+        # there are actually 3 different label-element in a layer element:
+        for element in labelElements:
+            labelParent = element.parentNode.localName
+            if labelParent == 'maplayer':
+                labelOn = element.childNodes[0].nodeValue.encode('utf-8')
+            if labelParent == 'labelattributes':
+                labelField = element.getAttribute('fieldname').encode('utf-8')
+        if labelField != '' and labelField is not None and labelOn == "1" and labelOn is not None:
           layer_def += "    LABELITEM '" + labelField + "'\n"
       except:
         # no labels
         pass
-      
+    
       # write the CLASS section for rendering
       # First see if there is a single symbol renderer
       if lyr.getElementsByTagName("singlesymbol").length > 0:
@@ -612,19 +630,23 @@ class Qgis2Map:
       # end of LAYER
       layer_def += "  END\n\n"
 
-      # add the layer to the list
-      layer_list[layer_name] = layer_def
-    # all layers have been processed, reverse the list and write
-    # not necessary since z-order is mapped by the legend list order
-    self.z_order.reverse()
-    for layer in self.z_order:
-      self.outFile.write(layer_list[layer])
+      # add the layer to the list with layer_id as key
+      layer_list[layer_id] = layer_def
+    # all layers have been processed, reverse the layer_order and write
+    # output layer_def's in order as they appear in legend (as seen by user)
+    self.layer_order.reverse()
+    for layerid in self.layer_order:
+      self.outFile.write(layer_list[layerid])
     return resultMsg
 
 
   def getEpsg(self, lyr):
-    srs = lyr.getElementsByTagName('srs')[0].getElementsByTagName('spatialrefsys')[0]
-    return srs.getElementsByTagName('epsg')[0].childNodes[0].nodeValue.encode('utf-8')
+    try:
+      srs = lyr.getElementsByTagName('srs')[0].getElementsByTagName('spatialrefsys')[0]
+      return srs.getElementsByTagName('srid')[0].childNodes[0].nodeValue.encode('utf-8')
+    except:  
+      #Use 4326 as a sensible default if the above fails
+      return "4326"
 
 
   def getPrimaryKey(self, layerId, tableName):
@@ -700,6 +722,61 @@ class Qgis2Map:
     # using a mapfile pre-processor, the proper id field can be substituted in the following:
     return str("%" + tableName + "_id%")
 
+
+  # Get RGB code from a XML color node, returning string like '45 124 255'
+  def getRgbFromNode(self, symbolNode, elementName):
+    if symbolNode == None or symbolNode.getElementsByTagName(elementName).length == 0:
+        return ''
+    colorNode = symbolNode.getElementsByTagName(elementName)[0]
+    return colorNode.getAttribute('red').encode('utf-8') + ' ' + colorNode.getAttribute('green').encode('utf-8') + ' ' + colorNode.getAttribute('blue').encode('utf-8')
+
+
+  # Get a size (multiplied by symbolSizeMultiplier) from given sizeNode's child with elementName 
+  def getSizeStringFromNode(self, symbolNode, elementName):
+    if symbolNode == None or symbolNode.getElementsByTagName(elementName).length == 0:
+        return ''
+    size = float(symbolNode.getElementsByTagName(elementName)[0].childNodes[0].nodeValue.encode('utf-8'))
+    symbolSize = size * symbolSizeMultiplier
+    return str(symbolSize)
+
+
+  def getSymbolProperty(self, symbolNode, elementName):
+    if symbolNode == None or symbolNode.getElementsByTagName(elementName).length == 0:
+        return ''
+    return str(symbolNode.getElementsByTagName(elementName)[0].childNodes[0].nodeValue.encode('utf-8'))
+
+
+  def writeClassStyleContent(self, symbolNode, geometry):
+    outlinecolor = self.getRgbFromNode(symbolNode, 'outlinecolor')
+    fillcolor    = self.getRgbFromNode(symbolNode, 'fillcolor')
+    pointsize    = self.getSizeStringFromNode(symbolNode, 'pointsize')
+    outlinewidth = self.getSizeStringFromNode(symbolNode, 'outlinewidth')
+    fillpattern  = self.getSymbolProperty(symbolNode, 'fillpattern');
+
+    class_def = "       STYLE\n"
+    # for POINT by SYMBOL and SIZE
+    if geometry == 'POINT':
+        # use the point symbol map to lookup the mapserver symbol type
+        symbol = self.msSymbol( geometry, symbolNode )
+        class_def += "         SYMBOL " + symbol + " \n"
+        class_def += "         SIZE " + pointsize + " \n"
+    # for LINE and POLYGON size define by WIDTH
+    else:
+        class_def += "         WIDTH " + outlinewidth + " \n"
+
+    # for LINE defined by COLOR from outlinecolor
+    if geometry == 'LINE':
+        class_def += "         COLOR " + outlinecolor + "\n"
+    # for POLYGON and POINT defined by COLOR from fillcolor and OUTLINECOLOR from outlinecolor
+    else:
+        class_def += "         OUTLINECOLOR " + outlinecolor + "\n"
+        if 'NoBrush' != fillpattern:
+            class_def += "         COLOR " + fillcolor + "\n"
+
+    class_def += "       END\n"
+    return class_def
+
+
   # Simple renderer ouput
   # We need the layer node and symbol node
   def simpleRenderer(self, layerNode, symbolNode):
@@ -710,20 +787,7 @@ class Qgis2Map:
 
     class_def += "       NAME '" + layerNode.getElementsByTagName("layername")[0].childNodes[0].nodeValue.encode('utf-8').replace("\"", "") + "' \n"
 
-    class_def += "       STYLE\n"
-    # use the point symbol map to lookup the mapserver symbol type
-    symbol = self.msSymbol( geometry, symbolNode )
-    class_def += "         SYMBOL " + symbol + " \n"
-    class_def += self.getSymbolSizeString(symbolNode)
-
-    # outline color
-    outlineNode = symbolNode.getElementsByTagName('outlinecolor')[0]
-    class_def += "         OUTLINECOLOR " + outlineNode.getAttribute('red').encode('utf-8') + ' ' + outlineNode.getAttribute('green').encode('utf-8') + ' ' + outlineNode.getAttribute('blue').encode('utf-8') + "\n"
-    # color
-    colorNode = symbolNode.getElementsByTagName('fillcolor')[0]
-    class_def += "         COLOR " + colorNode.getAttribute('red').encode('utf-8') + ' ' + colorNode.getAttribute('green').encode('utf-8') + ' ' + colorNode.getAttribute('blue').encode('utf-8') + "\n"
-
-    class_def += "       END\n"
+    class_def += self.writeClassStyleContent(symbolNode, geometry)
 
     class_def += self.msLabel( layerNode ) 
 
@@ -733,12 +797,6 @@ class Qgis2Map:
     return class_def
 
 
-  def getSymbolSizeString(self, symbolNode):
-    size = float(symbolNode.getElementsByTagName('pointsize')[0].childNodes[0].nodeValue.encode('utf-8'))
-    symbolSize = size * symbolSizeMultiplier
-    return "         SIZE " + str(symbolSize) + " \n"
-
-
   # Graduated symbol renderer output
   def graduatedRenderer(self, layerNode, symbolNode):
     # get the layers geometry type
@@ -766,22 +824,7 @@ class Qgis2Map:
 
       class_def += "      EXPRESSION ( ([" + classField + "] >= " + lower + ") AND ([" + classField + "] <= " + upper + ") )\n"
 
-      class_def += "      STYLE\n"
-      symbol = self.msSymbol( geometry, symbolNode )
-      class_def += "        SYMBOL " + symbol + "\n"
-
-      # Symbol size 
-      if geometry == 'POINT' or geometry == 'LINE':
-        class_def += self.getSymbolSizeString(cls)
-
-      # outline color
-      outlineNode = cls.getElementsByTagName('outlinecolor')[0]
-      class_def += "          OUTLINECOLOR " + outlineNode.getAttribute('red').encode('utf-8') + ' ' + outlineNode.getAttribute('green').encode('utf-8') + ' ' + outlineNode.getAttribute('blue').encode('utf-8') + "\n"
-      # color
-      colorNode = cls.getElementsByTagName('fillcolor')[0]
-      class_def += "          COLOR " + colorNode.getAttribute('red').encode('utf-8') + ' ' + colorNode.getAttribute('green').encode('utf-8') + ' ' + colorNode.getAttribute('blue').encode('utf-8') + "\n"
-
-      class_def += "        END\n"
+      class_def += self.writeClassStyleContent(cls, geometry)
 
       # label
       class_def += self.msLabel( layerNode ) 
@@ -791,6 +834,7 @@ class Qgis2Map:
 
     return class_def
 
+
   # Continuous symbol renderer output
   def continuousRenderer(self, layerNode, symbolNode):
     # get the layers geometry type
@@ -808,8 +852,8 @@ class Qgis2Map:
     # color
     lower = symbolNode.getElementsByTagName('lowestsymbol')[0].getElementsByTagName('symbol')[0]
     upper = symbolNode.getElementsByTagName('highestsymbol')[0].getElementsByTagName('symbol')[0]
-    lowerColor = lower.getElementsByTagName('fillcolor')[0]
-    upperColor = upper.getElementsByTagName('fillcolor')[0]
+    lowerColor = lower.getElementsByTagName('outlinecolor')[0]
+    upperColor = upper.getElementsByTagName('outlinecolor')[0]
 
     # outline color
     outlineNode = lower.getElementsByTagName('outlinecolor')[0]
@@ -823,11 +867,43 @@ class Qgis2Map:
     class_def += "        DATARANGE " + lower.getElementsByTagName('lowervalue')[0].childNodes[0].nodeValue.encode('utf-8') + ' ' + upper.getElementsByTagName('lowervalue')[0].childNodes[0].nodeValue.encode('utf-8') + '\n'
 
     class_def += "        RANGEITEM '" + classField + "'\n"
-    class_def += "      END\n"
 
-    class_def += "      STYLE\n"
-    class_def += "        OUTLINECOLOR " + outlineNode.getAttribute('red').encode('utf-8') + " " + outlineNode.getAttribute('green').encode('utf-8') + " " + outlineNode.getAttribute('blue').encode('utf-8') + "\n"
+
+    # upper and lower have same size
+    outlinecolor = self.getRgbFromNode(upper, 'outlinecolor')
+    fillcolor    = self.getRgbFromNode(upper, 'fillcolor')
+    outlinewidth = self.getSizeStringFromNode(upper, 'outlinewidth')
+
+    size = float(lower.getElementsByTagName('pointsize')[0].childNodes[0].nodeValue.encode('utf-8'))
+    pointsize = 2 * size * symbolSizeMultiplier
+
+
+    # for POINT by SYMBOL and SIZE
+    if geometry == 'POINT':
+        # use the point symbol map to lookup the mapserver symbol type
+        symbol = self.msSymbol( geometry, symbolNode )
+        class_def += "         SYMBOL " + symbol + " \n"
+        class_def += "         SIZE " + str(pointsize) + " \n"
+    # for LINE and POLYGON size define by WIDTH
+    else:
+        class_def += "         WIDTH " + outlinewidth + " \n"
+
+    # for LINE defined by COLOR from outlinecolor
+    if geometry == 'LINE':
+        class_def += "         COLOR " + outlinecolor + "\n"
+    # for POLYGON and POINT defined by COLOR from fillcolor and OUTLINECOLOR from outlinecolor
+    else:
+        class_def += "         COLOR " + fillcolor + "\n"
+
     class_def += "      END\n"
+    
+    # only outlines for polygons
+    outline = symbolNode.getElementsByTagName('polygonoutline')
+    if geometry == 'POLYGON' and outline is not None and '1' == outline[0].childNodes[0].nodeValue.encode('utf-8'):
+      class_def += "     STYLE\n"
+      class_def += "       WIDTH 1 " + "\n"
+      class_def += "       OUTLINECOLOR 0 0 0" + "\n"
+      class_def += "     END\n"
 
     # label
     class_def +=  self.msLabel( layerNode )
@@ -836,7 +912,7 @@ class Qgis2Map:
     class_def += "    END\n"
 
     return class_def
-    
+
 
   # Unique value renderer output
   def uniqueRenderer(self, layerNode, symbolNode):
@@ -863,47 +939,23 @@ class Qgis2Map:
       # If there's a label use it, otherwise autogenerate one
       try:
         label = cls.getElementsByTagName('label')[0].childNodes[0].nodeValue.encode('utf-8')
-        class_def += "      NAME '" + label + "'\n"
+        class_def += '      NAME "' + label + '"\n'
       except:
-        class_def += "      NAME '" + classField + " = " + lower + "' \n"
+        class_def += '      NAME "' + classField + ' = ' + lower + '" \n'
 
-      class_def += "      EXPRESSION '" + lower + "' \n"
+      class_def += '      EXPRESSION "' + lower + '" \n'
 
-      # Get the symbol name
-      symbol = self.msSymbol( geometry, symbolNode )  
-      
-      class_def += "      STYLE\n"
-      class_def += "        SYMBOL " + symbol + "\n"
-
-      # Symbol size 
-      if geometry == 'POINT' or geometry == 'LINE':
-        class_def += self.getSymbolSizeString(cls)
-
-      # outline color
-      outlineNode = cls.getElementsByTagName('outlinecolor')[0]
-      class_def += "         OUTLINECOLOR "  \
-            + outlineNode.getAttribute('red').encode('utf-8') + ' ' \
-            + outlineNode.getAttribute('green').encode('utf-8') + ' ' \
-            + outlineNode.getAttribute('blue').encode('utf-8') \
-            + "\n"
-
-      # color
-      colorNode = cls.getElementsByTagName('fillcolor')[0]
-      class_def += "         COLOR "  \
-            + colorNode.getAttribute('red').encode('utf-8') + ' ' \
-            + colorNode.getAttribute('green').encode('utf-8') + ' ' \
-            + colorNode.getAttribute('blue').encode('utf-8') \
-            + "\n"
-      class_def += "       END\n"
+      class_def += self.writeClassStyleContent(cls, geometry)
 
       # label
       class_def +=  self.msLabel( layerNode )
-      
+
       # end of CLASS  
       class_def += "    END\n"
 
     return class_def
-    
+
+
   # Utility method to format a proj4 text string into mapserver format
   def formatProj4(self, proj4text):
     parms = proj4text.split(" ")
@@ -913,6 +965,7 @@ class Qgis2Map:
       ret = ret + "    '" + p + "'\n"
     return ret
 
+
   def getProj4(self, proj4text):
     """Returns the proj4 string as a dictionary with key value pairs."""
     parms = proj4text.split(" ")
@@ -931,6 +984,7 @@ class Qgis2Map:
         ret[key] = value
     return ret
 
+
   # Determines the symbol name and adds it to the symbol queue
   def msSymbol(self, geometry, symbolNode):
     # contains the same markup for a layer regardless of type
@@ -939,9 +993,11 @@ class Qgis2Map:
     symbol = '0'
 
     if geometry == 'POLYGON':
-      symbol = '0'
+      #symbol = '0'
+      pass
     elif geometry == 'LINE':
-      symbol = '0'
+      #symbol = '0'
+      pass
     elif geometry == 'POINT':
       try:
         symbolName = qgis2map_symbol[symbolNode.getElementsByTagName('pointsymbol')[0].childNodes[0].nodeValue.encode('utf-8')]
@@ -1064,7 +1120,12 @@ class Qgis2Map:
       # qgis uses a fill color around the label
       # Note that buffer only works for truetype fonts
       buffer = labelNode.getElementsByTagName('buffersize')[0].getAttribute('value').encode('utf-8')
-      labelBlock += "      BUFFER " + buffer + "\n"
+      bufferon = labelNode.getElementsByTagName('bufferenabled')[0].getAttribute('on').encode('utf-8')
+      buffercolor= self.getRgbFromNode(labelNode, 'buffercolor')
+      if bufferon is not None and '1' == bufferon:
+        # not sure if we should use this BUFFER
+        #labelBlock += "      BUFFER " + buffer + "\n"
+        labelBlock += "      BACKGROUNDCOLOR " + buffercolor + "\n"
 
       # alignment in QGis corresponds to position in MapServer
       alignment = labelNode.getElementsByTagName('alignment')[0].getAttribute('value').encode('utf-8')
