@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """
 Copyright (C) 2007-2008 Matthew Perry
-Copyright (C) 2008 Borys Jurgiel
+Copyright (C) 2008-2010 Borys Jurgiel
 
 /***************************************************************************
  *                                                                         *
@@ -20,7 +20,7 @@ from PyQt4.QtNetwork import *
 from qgis.core import *
 from unzip import unzip
 from version_compare import compareVersions, normalizeVersion
-
+import sys
 
 """
 Data structure:
@@ -29,7 +29,7 @@ mRepositories = dict of dicts: {repoName : {"url" QString,
                                             "valid" bool,
                                             "QPHttp" QPHttp,
                                             "Relay" Relay, # Relay object for transmitting signals from QPHttp with adding the repoName information
-                                            "xmlData" QDomDocument,
+                                            "xmlData" QBuffer,
                                             "state" int,   (0 - disabled, 1-loading, 2-loaded ok, 3-error (to be retried), 4-rejected)
                                             "error" QString}}
 mPlugins = dict of dicts {id : {"name" QString,
@@ -43,6 +43,7 @@ mPlugins = dict of dicts {id : {"name" QString,
                                 "error_details" QString,
                                 "homepage" QString,
                                 "url" QString,
+                                "experimental" bool
                                 "filename" QString,
                                 "repository" QString,
                                 "localdir" QString,
@@ -50,15 +51,9 @@ mPlugins = dict of dicts {id : {"name" QString,
 """
 
 
-try:
-  QGIS_VER = QGis.qgisVersion
-  if QGIS_VER[0] == "1":
-    QGIS_MAJOR_VER = 1
-  else:
-    QGIS_MAJOR_VER = 0
-except:
-  QGIS_VER = QGis.QGIS_VERSION
-  QGIS_MAJOR_VER = 1
+QGIS_VER = QGis.QGIS_VERSION
+QGIS_14 = (QGIS_VER[2] > "3")
+QGIS_15 = (QGIS_VER[2] > "4")
 
 
 
@@ -74,16 +69,55 @@ seenPluginGroup = "/Qgis/plugin-seen"
 
 
 # Repositories: (name, url, possible depreciated url)
-oldRepo      = ("QGIS 0.x Plugin Repository",  "http://spatialserver.net/cgi-bin/pyqgis_plugin.rb","")
 officialRepo = ("QGIS Official Repository",    "http://pyqgis.org/repo/official","")
 contribRepo  = ("QGIS Contributed Repository", "http://pyqgis.org/repo/contributed","")
-authorRepos  = [("Carson Farmer's Repository", "http://www.ftools.ca/cfarmerQgisRepo.xml", "http://www.ftools.ca/cfarmerQgisRepo_0.xx.xml"),
-                ("Borys Jurgiel's Repository", "http://bwj.aster.net.pl/qgis/plugins.xml", "http://bwj.aster.net.pl/qgis-oldapi/plugins.xml"),
-                ("Faunalia Repository",        "http://www.faunalia.it/qgis/plugins.xml",  "http://faunalia.it/qgis/plugins.xml"),
-                ("Martin Dobias' Sandbox",     "http://mapserver.sk/~wonder/qgis/plugins-sandbox.xml", ""),
-                ("Aaron Racicot's Repository", "http://qgisplugins.z-pulley.com", ""),
+authorRepos  = [("Aaron Racicot's Repository", "http://qgisplugins.z-pulley.com", ""),
                 ("Barry Rowlingson's Repository", "http://www.maths.lancs.ac.uk/~rowlings/Qgis/Plugins/plugins.xml", ""),
-                ("GIS-Lab Repository",         "http://gis-lab.info/programs/qgis/qgis-repo.xml", "")]
+                # depreciated: ("Bob Bruce's Repository",     "http://www.mappinggeek.ca/QGISPythonPlugins/Bobs-QGIS-plugins.xml", ""),
+                # depreciated: ("Borys Jurgiel's Repository", "http://bwj.aster.net.pl/qgis/plugins.xml", "http://bwj.aster.net.pl/qgis-oldapi/plugins.xml"),
+                ("Carson Farmer's Repository", "http://www.ftools.ca/cfarmerQgisRepo.xml", "http://www.ftools.ca/cfarmerQgisRepo_0.xx.xml"),
+                ("CatAIS Repository",          "http://www.catais.org/qgis/plugins.xml", ""),
+                ("Faunalia Repository",        "http://www.faunalia.it/qgis/plugins.xml", "http://faunalia.it/qgis/plugins.xml"),
+                ("GIS-Lab Repository",         "http://gis-lab.info/programs/qgis/qgis-repo.xml", ""),
+                ("Martin Dobias' Sandbox",     "http://mapserver.sk/~wonder/qgis/plugins-sandbox.xml", ""),
+                ("Marco Hugentobler's Repository","http://karlinapp.ethz.ch/python_plugins/python_plugins.xml", ""),
+                ("Sourcepole Repository",      "http://build.sourcepole.ch/qgis/plugins.xml", ""),
+                ("Volkan Kepoglu's Repository","http://ggit.metu.edu.tr/~volkan/plugins.xml", "")]
+
+
+
+# --- class History  ----------------------------------------------------------------------- #
+class History(dict):
+  """ Dictionary for keeping changes made duging the session - will be used to complete the (un/re)loading """
+
+  # ----------------------------------------- #
+  def markChange(self, plugin, change):
+    """ mark the status change: A-added, D-deleted, R-reinstalled """
+    if change == "A" and self.get(plugin) == "D":   # installation right after uninstallation
+      self.__setitem__(plugin, "R")
+    elif change == "A":                             # any other installation
+      self.__setitem__(plugin, "A")
+    elif change == "D" and self.get(plugin) == "A": # uninstallation right after installation
+      self.pop(plugin)
+    elif change == "D":                             # any other uninstallation
+      self.__setitem__(plugin, "D")
+    elif change == "R" and self.get(plugin) == "A": # reinstallation right after installation
+      self.__setitem__(plugin, "A")
+    elif change == "R":                             # any other reinstallation
+      self.__setitem__(plugin, "R")
+
+  # ----------------------------------------- #
+  def toList(self, change):
+    """ return a list of plugins matching the given change """
+    result = []
+    for i in self.items():
+      if i[1] == change:
+        result += [i[0]]
+    return result
+# --- /class History  ---------------------------------------------------------------------- #
+
+
+
 
 
 # --- class QPHttp  ----------------------------------------------------------------------- #
@@ -96,6 +130,8 @@ class QPHttp(QHttp):
     if settings.value("/proxyEnabled").toBool():
       self.proxy=QNetworkProxy()
       proxyType = settings.value( "/proxyType", QVariant(0)).toString()
+      if len(args)>0 and settings.value("/proxyExcludedUrls").toString().contains(args[0]):
+        proxyType = "NoProxy"
       if proxyType in ["1","Socks5Proxy"]: self.proxy.setType(QNetworkProxy.Socks5Proxy)
       elif proxyType in ["2","NoProxy"]: self.proxy.setType(QNetworkProxy.NoProxy)
       elif proxyType in ["3","HttpProxy"]: self.proxy.setType(QNetworkProxy.HttpProxy)
@@ -159,17 +195,12 @@ class Repositories(QObject):
     settings = QSettings()
     settings.beginGroup(reposGroup)
     # add the central repositories
-    if QGIS_MAJOR_VER: # QGIS 1.x
-      if presentURLs.count(officialRepo[1]) == 0:
-        settings.setValue(officialRepo[0]+"/url", QVariant(officialRepo[1]))
-        settings.setValue(officialRepo[0]+"/enabled", QVariant(True))
-      if presentURLs.count(contribRepo[1]) == 0:
-        settings.setValue(contribRepo[0]+"/url", QVariant(contribRepo[1]))
-        settings.setValue(contribRepo[0]+"/enabled", QVariant(True))
-    else: # QGIS 0.x
-      if presentURLs.count(oldRepo[1]) == 0:
-        settings.setValue(oldRepo[0]+"/url", QVariant(oldRepo[1]))
-        settings.setValue(oldRepo[0]+"/enabled", QVariant(True))
+    if presentURLs.count(officialRepo[1]) == 0:
+      settings.setValue(officialRepo[0]+"/url", QVariant(officialRepo[1]))
+      settings.setValue(officialRepo[0]+"/enabled", QVariant(True))
+    if presentURLs.count(contribRepo[1]) == 0:
+      settings.setValue(contribRepo[0]+"/url", QVariant(contribRepo[1]))
+      settings.setValue(contribRepo[0]+"/enabled", QVariant(True))
     # add author repositories
     for i in authorRepos:
       if i[1] and presentURLs.count(i[1]) == 0:
@@ -207,11 +238,17 @@ class Repositories(QObject):
 
 
   # ----------------------------------------- #
-  def setRepositoryData(self,reposName, key, value):
+  def setRepositoryData(self, reposName, key, value):
     """ write data to the mRepositories dict """
     self.mRepositories[reposName][key] = value
 
 
+  # ----------------------------------------- #
+  def remove(self, reposName):
+    """ remove given item from the mRepositories dict """
+    del self.mRepositories[reposName]
+
+
   # ----------------------------------------- #
   def rename(self, oldName, newName):
     """ rename repository key """
@@ -235,6 +272,48 @@ class Repositories(QObject):
     settings.setValue(settingsGroup+"/checkOnStart", QVariant(state))
 
 
+  # ----------------------------------------- #
+  def checkingOnStartInterval(self):
+    """ return checking for news and updates interval """
+    settings = QSettings()
+    (i, ok) = settings.value(settingsGroup+"/checkOnStartInterval").toInt()
+    if i < 0 or not ok:
+      i = 1
+    # allowed values: 0,1,3,7,14,30 days
+    interval = 0
+    for j in [1,3,7,14,30]:
+      if i >= j:
+        interval = j
+    return interval
+
+
+  # ----------------------------------------- #
+  def setCheckingOnStartInterval(self, interval):
+    """ set checking for news and updates interval """
+    settings = QSettings()
+    settings.setValue(settingsGroup+"/checkOnStartInterval", QVariant(interval))
+
+
+  # ----------------------------------------- #
+  def saveCheckingOnStartLastDate(self):
+    """ set today's date as the day of last checking  """
+    settings = QSettings()
+    settings.setValue(settingsGroup+"/checkOnStartLastDate", QVariant(QDate.currentDate()))
+
+
+  # ----------------------------------------- #
+  def timeForChecking(self):
+    """ determine whether it's the time for checking for news and updates now """
+    if self.checkingOnStartInterval() == 0:
+      return True
+    settings = QSettings()
+    interval = settings.value(settingsGroup+"/checkOnStartLastDate").toDate().daysTo(QDate.currentDate())
+    if interval >= self.checkingOnStartInterval():
+      return True
+    else:
+      return False
+
+
   # ----------------------------------------- #
   def load(self):
     """ populate the mRepositories dict"""
@@ -242,31 +321,20 @@ class Repositories(QObject):
     settings = QSettings()
     settings.beginGroup(reposGroup)
     # first, update repositories in QSettings if needed
-    if QGIS_MAJOR_VER:
-      mainRepo = officialRepo
-      invalidRepo = oldRepo
-    else:
-      mainRepo = oldRepo
-      invalidRepo = officialRepo
-    mainRepoPresent = False
+    officialRepoPresent = False
     for key in settings.childGroups():
       url = settings.value(key+"/url", QVariant()).toString()
       if url == contribRepo[1]:
-        if QGIS_MAJOR_VER:
-          settings.setValue(key+"/valid", QVariant(True)) # unlock the contrib repo in qgis 1.x
-        else:
-          settings.setValue(key+"/valid", QVariant(False)) # lock the contrib repo in qgis 0.x
+        settings.setValue(key+"/valid", QVariant(True)) # unlock the contrib repo in qgis 1.x
       else:
         settings.setValue(key+"/valid", QVariant(True)) # unlock any other repo
-      if url == mainRepo[1]:
-        mainRepoPresent = True
-      if url == invalidRepo[1]:
-        settings.remove(key)
+      if url == officialRepo[1]:
+        officialRepoPresent = True
       for authorRepo in authorRepos:
         if url == authorRepo[2]:
           settings.setValue(key+"/url", QVariant(authorRepo[1])) # correct a depreciated url
-    if not mainRepoPresent:
-      settings.setValue(mainRepo[0]+"/url", QVariant(mainRepo[1]))
+    if not officialRepoPresent:
+      settings.setValue(officialRepo[0]+"/url", QVariant(officialRepo[1]))
 
     for key in settings.childGroups():
       self.mRepositories[key] = {}
@@ -287,7 +355,10 @@ class Repositories(QObject):
     self.mRepositories[key]["state"] = 1
     url = QUrl(self.mRepositories[key]["url"])
     path = QString(url.toPercentEncoding(url.path(), "!$&'()*+,;=:@/"))
-    self.mRepositories[key]["QPHttp"] = QPHttp(url.host())
+    port = url.port()
+    if port < 0:
+      port = 80
+    self.mRepositories[key]["QPHttp"] = QPHttp(url.host(), port)
     self.connect(self.mRepositories[key]["QPHttp"], SIGNAL("requestFinished (int, bool)"), self.xmlDownloaded)
     self.connect(self.mRepositories[key]["QPHttp"], SIGNAL("stateChanged ( int )"), self.mRepositories[key]["Relay"].stateChanged)
     self.connect(self.mRepositories[key]["QPHttp"], SIGNAL("dataReadProgress ( int , int )"), self.mRepositories[key]["Relay"].dataReadProgress)
@@ -328,18 +399,23 @@ class Repositories(QObject):
       pluginNodes = reposXML.elementsByTagName("pyqgis_plugin")
       if pluginNodes.size():
         for i in range(pluginNodes.size()):
-          fileName = QFileInfo(pluginNodes.item(i).firstChildElement("download_url").text().trimmed()).fileName()
+          fileName = pluginNodes.item(i).firstChildElement("file_name").text().simplified()
+          if not fileName:
+              fileName = QFileInfo(pluginNodes.item(i).firstChildElement("download_url").text().trimmed().split("?")[0]).fileName()
           name = fileName.section(".", 0, 0)
-          name = str(name)
-          plugin = {}
-          plugin[name] = {
+          name = unicode(name)
+          experimental = False
+          if pluginNodes.item(i).firstChildElement("experimental").text().simplified().toUpper() in ["TRUE","YES"]:
+            experimental = True
+          plugin = {
             "name"          : pluginNodes.item(i).toElement().attribute("name"),
             "version_avail" : pluginNodes.item(i).toElement().attribute("version"),
-            "desc_repo"     : pluginNodes.item(i).firstChildElement("description").text().trimmed(),
+            "desc_repo"     : pluginNodes.item(i).firstChildElement("description").text().simplified(),
             "desc_local"    : "",
-            "author"        : pluginNodes.item(i).firstChildElement("author_name").text().trimmed(),
-            "homepage"      : pluginNodes.item(i).firstChildElement("homepage").text().trimmed(),
-            "url"           : pluginNodes.item(i).firstChildElement("download_url").text().trimmed(),
+            "author"        : pluginNodes.item(i).firstChildElement("author_name").text().simplified(),
+            "homepage"      : pluginNodes.item(i).firstChildElement("homepage").text().simplified(),
+            "url"           : pluginNodes.item(i).firstChildElement("download_url").text().simplified(),
+            "experimental"  : experimental,
             "filename"      : fileName,
             "status"        : "not installed",
             "error"         : "",
@@ -348,16 +424,17 @@ class Repositories(QObject):
             "repository"    : reposName,
             "localdir"      : name,
             "read-only"     : False}
-          qgisMinimumVersion = pluginNodes.item(i).firstChildElement("qgis_minimum_version").text().trimmed()
+          qgisMinimumVersion = pluginNodes.item(i).firstChildElement("qgis_minimum_version").text().simplified()
           if not qgisMinimumVersion: qgisMinimumVersion = "0"
           # please use the tag below only if really needed! (for example if plugin development is abandoned)
-          qgisMaximumVersion = pluginNodes.item(i).firstChildElement("qgis_maximum_version").text().trimmed()
+          qgisMaximumVersion = pluginNodes.item(i).firstChildElement("qgis_maximum_version").text().simplified()
           if not qgisMaximumVersion: qgisMaximumVersion = "2"
           #if compatible, add the plugin to the list
-          if compareVersions(QGIS_VER, qgisMinimumVersion) < 2 and compareVersions(qgisMaximumVersion, QGIS_VER) < 2:
-            if QGIS_VER[0]=="0" or qgisMinimumVersion[0]=="1" or name=="plugin_installer":
-              plugins.addPlugin(plugin)
-        plugins.workarounds()
+          if not pluginNodes.item(i).firstChildElement("disabled").text().simplified().toUpper() in ["TRUE","YES"]:
+           if compareVersions(QGIS_VER, qgisMinimumVersion) < 2 and compareVersions(qgisMaximumVersion, QGIS_VER) < 2:
+            if QGIS_VER[0]==qgisMinimumVersion[0] or (qgisMinimumVersion!="0" and qgisMaximumVersion!="2"):   # to be deleted
+              #add the plugin to the cache
+              plugins.addFromRepository(plugin)
         self.mRepositories[reposName]["state"] = 2
       else:
         #print "Repository parsing error"
@@ -368,7 +445,8 @@ class Repositories(QObject):
 
     # is the checking done?
     if not self.fetchingInProgress():
-      plugins.getAllInstalled()
+      plugins.rebuild()
+      self.saveCheckingOnStartLastDate()
       self.emit(SIGNAL("checkingDone()"))
 # --- /class Repositories ---------------------------------------------------------------- #
 
@@ -382,7 +460,10 @@ class Plugins(QObject):
   # ----------------------------------------- #
   def __init__(self):
     QObject.__init__(self)
-    self.mPlugins = {}
+    self.mPlugins = {}   # the dict of plugins (dicts)
+    self.repoCache = {}  # the dict of lists of plugins (dicts)
+    self.localCache = {} # the dict of plugins (dicts)
+    self.obsoletePlugins = [] # the list of outdated 'user' plugins masking newer 'system' ones
 
 
   # ----------------------------------------- #
@@ -401,37 +482,39 @@ class Plugins(QObject):
 
 
   # ----------------------------------------- #
-  def setPluginData(self, pluginName, key, value):
-    """ write data to the mPlugins dict """
-    self.mPlugins[pluginName][key] = value
+  def addInstalled(self, plugin):
+    """ add given plugin to the localCache """
+    key = plugin["localdir"]
+    self.localCache[key] = plugin
 
 
   # ----------------------------------------- #
-  def clear(self):
-    """ clear the plugins dict"""
-    self.mPlugins = {}
+  def addFromRepository(self, plugin):
+    """ add given plugin to the repoCache """
+    repo = plugin["repository"]
+    try:
+      self.repoCache[repo] += [plugin]
+    except:
+      self.repoCache[repo] = [plugin]
 
 
   # ----------------------------------------- #
-  def addPlugin(self, plugins):
-    """ add a plugin (first from given) to the mPlugins dict """
-    key = plugins.keys()[0]
-    plugin = plugins[key]
-    plugin["version_avail"] = normalizeVersion(plugin["version_avail"])
-    plugin["version_inst"] = normalizeVersion(plugin["version_inst"])
-    if not self.mPlugins.has_key(key) or compareVersions(self.mPlugins[key]["version_avail"],plugin["version_avail"]) == 2:
-      self.mPlugins[key] = plugin # add the plugin if not present yet or if is newer than existing one
+  def removeInstalledPlugin(self, key):
+    """ remove given plugin from the localCache """
+    if self.localCache.has_key(key):
+      del self.localCache[key]
 
 
   # ----------------------------------------- #
-  def remove(self, key):
-    """ remove given plugin from the mPlugins dict """
-    del self.mPlugins[key]
+  def removeRepository(self, repo):
+    """ remove whole repository from the repoCache """
+    if self.repoCache.has_key(repo):
+      del self.repoCache[repo]
 
 
   # ----------------------------------------- #
-  def updatePlugin(self, key, readOnly):
-    """ The mPlugins should contain available plugins first. Now, add installed one (add when not present, update if present) """
+  def getInstalledPlugin(self, key, readOnly):
+    """ get the metadata of an installed plugin """
     if readOnly:
       path = QgsApplication.pkgDataPath()
     else:
@@ -447,7 +530,8 @@ class Plugins(QObject):
     error = ""
     errorDetails = ""
     try:
-      exec("import "+ key)
+      exec("import %s" % key)
+      exec("reload (%s)" % key)
       try:
         exec("nam = %s.name()" % key)
       except:
@@ -478,9 +562,9 @@ class Plugins(QObject):
       try:
         exec ("%s.classFactory(iface)" % key)
       except Exception, error:
-        error = error.message
+        error = unicode(error.args[0])
     except Exception, error:
-      error = error.message
+      error = unicode(error.args[0])
 
     if not nam:
       nam = key
@@ -502,66 +586,40 @@ class Plugins(QObject):
         "author"        : auth,
         "homepage"      : homepage,
         "url"           : path,
+        "experimental"  : False,
         "filename"      : "",
-        "status"        : "",
+        "status"        : "orphan",
         "error"         : error,
         "error_details" : errorDetails,
         "repository"    : "",
         "localdir"      : key,
         "read-only"     : readOnly}
-
-    if not self.mPlugins.has_key(key):
-      self.mPlugins[key] = plugin   # just add a new plugin
-    else:
-      self.mPlugins[key]["localdir"] = plugin["localdir"]
-      self.mPlugins[key]["read-only"] = plugin["read-only"]
-      self.mPlugins[key]["error"] = plugin["error"]
-      self.mPlugins[key]["error_details"] = plugin["error_details"]
-      if plugin["name"] and plugin["name"] != key:
-        self.mPlugins[key]["name"] = plugin["name"] # local name has higher priority
-      self.mPlugins[key]["version_inst"] = plugin["version_inst"]
-      self.mPlugins[key]["desc_local"] = plugin["desc_local"]
-    # set status
-    #
-    # installed   available   status
-    # ---------------------------------------
-    # none        any         "not installed" (will be later checked if is "new")
-    # any         none        "orphan"
-    # same        same        "installed"
-    # less        greater     "upgradeable"
-    # greater     less        "newer"
-    if not self.mPlugins[key]["version_avail"]:
-      self.mPlugins[key]["status"] = "orphan"
-    elif self.mPlugins[key]["error"] in ["broken","dependent"]:
-      self.mPlugins[key]["status"] = "installed"
-    elif not self.mPlugins[key]["version_inst"]:
-      self.mPlugins[key]["status"] = "not installed"
-    elif compareVersions(self.mPlugins[key]["version_avail"],self.mPlugins[key]["version_inst"]) == 0:
-      self.mPlugins[key]["status"] = "installed"
-    elif compareVersions(self.mPlugins[key]["version_avail"],self.mPlugins[key]["version_inst"]) == 1:
-      self.mPlugins[key]["status"] = "upgradeable"
-    else:
-      self.mPlugins[key]["status"] = "newer"
+    return plugin
 
 
   # ----------------------------------------- #
   def getAllInstalled(self):
-    """ update the mPlugins dict with alredy installed plugins """
+    """ Build the localCache """
+    self.localCache = {}
     # first, try to add the read-only plugins...
+    pluginsPath = unicode(QDir.convertSeparators(QDir.cleanPath(QgsApplication.pkgDataPath() + "/python/plugins")))
+    #  temporarily add the system path as the first element to force loading the read-only plugins, even if masked by user ones.
+    sys.path = [pluginsPath] + sys.path
     try:
-      pluginDir = QDir.cleanPath(unicode(QgsApplication.pkgDataPath()) + "/python/plugins")
-      pluginDir = QDir(pluginDir)
+      pluginDir = QDir(pluginsPath)
       pluginDir.setFilter(QDir.AllDirs)
       for key in pluginDir.entryList():
         key = unicode(key)
         if not key in [".",".."]:
-          self.updatePlugin(key, True)
+          self.localCache[key] = self.getInstalledPlugin(key, True)
     except:
       # return QCoreApplication.translate("QgsPluginInstaller","Couldn't open the system plugin directory")
       pass # it's not necessary to stop due to this error
+    # remove the temporarily added path
+    sys.path.remove(pluginsPath)
     # ...then try to add locally installed ones
     try:
-      pluginDir = QDir.cleanPath(unicode(QgsApplication.qgisSettingsDirPath()) + "/python/plugins")
+      pluginDir = QDir.convertSeparators(QDir.cleanPath(QgsApplication.qgisSettingsDirPath() + "/python/plugins"))
       pluginDir = QDir(pluginDir)
       pluginDir.setFilter(QDir.AllDirs)
     except:
@@ -569,7 +627,69 @@ class Plugins(QObject):
     for key in pluginDir.entryList():
       key = unicode(key)
       if not key in [".",".."]:
-        self.updatePlugin(key, False)
+        plugin = self.getInstalledPlugin(key, False)
+        if key in self.localCache.keys() and compareVersions(self.localCache[key]["version_inst"],plugin["version_inst"]) == 1:
+          # An obsolete plugin in the "user" location is masking a newer one in the "system" location!
+          self.obsoletePlugins += [key]
+        self.localCache[key] = plugin
+
+
+  # ----------------------------------------- #
+  def rebuild(self):
+    """ build or rebuild the mPlugins from the caches """
+    self.mPlugins = {}
+    for i in self.localCache.keys():
+      self.mPlugins[i] = self.localCache[i].copy()
+    settings = QSettings()
+    (allowed, ok) = settings.value(settingsGroup+"/allowedPluginType", QVariant(2)).toInt()
+    for i in self.repoCache.values():
+      for plugin in i:
+        key = plugin["localdir"]
+        # check if the plugin is allowed and if there isn't any better one added already.
+        if (allowed != 1 or plugin["repository"] == officialRepo[0]) and (allowed == 3 or not plugin["experimental"]) \
+        and not (self.mPlugins.has_key(key) and self.mPlugins[key]["version_avail"] and compareVersions(self.mPlugins[key]["version_avail"], plugin["version_avail"]) < 2):
+          # The mPlugins doct contains now locally installed plugins. 
+          # Now, add the available one if not present yet or update it if present already.
+          if not self.mPlugins.has_key(key):
+            self.mPlugins[key] = plugin   # just add a new plugin
+          else:
+            self.mPlugins[key]["version_avail"] = plugin["version_avail"]
+            self.mPlugins[key]["desc_repo"] = plugin["desc_repo"]
+            self.mPlugins[key]["filename"] = plugin["filename"]
+            self.mPlugins[key]["repository"] = plugin["repository"]
+            self.mPlugins[key]["experimental"] = plugin["experimental"]
+            # use remote name if local one is not available
+            if self.mPlugins[key]["name"] == key and plugin["name"]:
+              self.mPlugins[key]["name"] = plugin["name"]
+            # those metadata has higher priority for their remote instances:
+            if plugin["author"]:
+              self.mPlugins[key]["author"] = plugin["author"]
+            if plugin["homepage"]:
+              self.mPlugins[key]["homepage"] = plugin["homepage"]
+            if plugin["url"]:
+              self.mPlugins[key]["url"] = plugin["url"]
+          # set status
+          #
+          # installed   available   status
+          # ---------------------------------------
+          # none        any         "not installed" (will be later checked if is "new")
+          # any         none        "orphan"
+          # same        same        "installed"
+          # less        greater     "upgradeable"
+          # greater     less        "newer"
+          if not self.mPlugins[key]["version_avail"]:
+            self.mPlugins[key]["status"] = "orphan"
+          elif self.mPlugins[key]["error"] in ["broken","dependent"]:
+            self.mPlugins[key]["status"] = "installed"
+          elif not self.mPlugins[key]["version_inst"]:
+            self.mPlugins[key]["status"] = "not installed"
+          elif compareVersions(self.mPlugins[key]["version_avail"],self.mPlugins[key]["version_inst"]) == 0:
+            self.mPlugins[key]["status"] = "installed"
+          elif compareVersions(self.mPlugins[key]["version_avail"],self.mPlugins[key]["version_inst"]) == 1:
+            self.mPlugins[key]["status"] = "upgradeable"
+          else:
+            self.mPlugins[key]["status"] = "newer"
+    self.markNews()
 
 
   # ----------------------------------------- #
@@ -603,18 +723,11 @@ class Plugins(QObject):
     return False
 
 
-  # ----------------------------------------- #
-  def workarounds(self):
-    """ workarounds for particular plugins with wrong metadata """
-    if self.mPlugins.has_key("select") and self.mPlugins["select"]["version_avail"] == "0.1":
-      self.mPlugins["select"]["version_avail"] = "0.2"
-
 # --- /class Plugins --------------------------------------------------------------------- #
 
 
 
-
-
 # public members:
+history = History()
 repositories = Repositories()
 plugins = Plugins()
