@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """
 Copyright (C) 2007-2008 Matthew Perry
-Copyright (C) 2008 Borys Jurgiel
+Copyright (C) 2008-2010 Borys Jurgiel
 
 /***************************************************************************
  *                                                                         *
@@ -18,13 +18,18 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from qgis.core import QgsApplication, QgsContextHelp
 import sys, time
-from qgsplugininstallerfetchingbase import Ui_QgsPluginInstallerFetchingDialogBase
-from qgsplugininstallerinstallingbase import Ui_QgsPluginInstallerInstallingDialogBase
-from qgsplugininstallerrepositorybase import Ui_QgsPluginInstallerRepositoryDetailsDialogBase
-from qgsplugininstallerpluginerrorbase import Ui_QgsPluginInstallerPluginErrorDialogBase
-from qgsplugininstallerbase import Ui_QgsPluginInstallerDialogBase
+from ui_qgsplugininstallerfetchingbase import Ui_QgsPluginInstallerFetchingDialogBase
+from ui_qgsplugininstallerinstallingbase import Ui_QgsPluginInstallerInstallingDialogBase
+from ui_qgsplugininstallerrepositorybase import Ui_QgsPluginInstallerRepositoryDetailsDialogBase
+from ui_qgsplugininstallerpluginerrorbase import Ui_QgsPluginInstallerPluginErrorDialogBase
+from ui_qgsplugininstallerbase import Ui_QgsPluginInstallerDialogBase
 from installer_data import *
 
+try:
+  from qgis.utils import startPlugin, unloadPlugin, loadPlugin # QGIS >= 1.4
+  from qgis.utils import reloadPlugin, updateAvailablePlugins # QGIS >= 1.5
+except Exception:
+  pass
 
 
 # --- common functions ------------------------------------------------------------------- #
@@ -152,7 +157,10 @@ class QgsPluginInstallerInstallingDialog(QDialog, Ui_QgsPluginInstallerInstallin
     tmpDir = QDir.tempPath()
     tmpPath = QDir.cleanPath(tmpDir+"/"+fileName)
     self.file = QFile(tmpPath)
-    self.http = QPHttp(url.host())
+    port = url.port()
+    if port < 0:
+      port = 80
+    self.http = QPHttp(url.host(), port)
     self.connect(self.http, SIGNAL("stateChanged ( int )"), self.stateChanged) 
     self.connect(self.http, SIGNAL("dataReadProgress ( int , int )"), self.readProgress)
     self.connect(self.http, SIGNAL("requestFinished (int, bool)"), self.requestFinished)
@@ -252,33 +260,46 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
     self.connect(self.treePlugins, SIGNAL("itemSelectionChanged()"), self.pluginTreeClicked)
     self.connect(self.treeRepositories, SIGNAL("itemSelectionChanged()"), self.repositoryTreeClicked)
     # buttons
-    self.connect(self.buttonInstall, SIGNAL("clicked()"), self.installPlugin)
-    self.connect(self.buttonUninstall, SIGNAL("clicked()"), self.uninstallPlugin)
+    self.connect(self.buttonInstall, SIGNAL("clicked()"), self.installPluginClicked)
+    self.connect(self.buttonUninstall, SIGNAL("clicked()"), self.uninstallPluginClicked)
     self.buttonInstall.setEnabled(False)
     self.buttonUninstall.setEnabled(False)
+    self.buttonHelp.setEnabled(QGIS_14)
+    self.connect(self.buttonHelp, SIGNAL("clicked()"), self.runHelp)
     # repositories handling
     self.connect(self.treeRepositories, SIGNAL("doubleClicked(QModelIndex)"), self.editRepository)
     self.connect(self.buttonFetchRepositories, SIGNAL("clicked()"), self.addKnownRepositories)
     self.connect(self.buttonAddRep, SIGNAL("clicked()"), self.addRepository)
     self.connect(self.buttonEditRep, SIGNAL("clicked()"), self.editRepository)
     self.connect(self.buttonDeleteRep, SIGNAL("clicked()"), self.deleteRepository)
-    # checkingOnStart checkbox
-    self.connect(self.checkUpdates, SIGNAL("stateChanged (int)"), self.ChangeCheckingPolicy)
-    if repositories.checkingOnStart():
-      self.checkUpdates.setCheckState(Qt.Checked)
-    else:
-      self.checkUpdates.setCheckState(Qt.Unchecked)
     self.buttonEditRep.setEnabled(False)
     self.buttonDeleteRep.setEnabled(False)
-
+    # configuration widgets
+    self.connect(self.checkUpdates, SIGNAL("toggled (bool)"), self.changeCheckingPolicy)
+    self.connect(self.comboInterval, SIGNAL("currentIndexChanged (int)"), self.changeCheckingInterval)
+    self.connect(self.radioPluginType0, SIGNAL("toggled (bool)"), self.changePluginPolicy)
+    self.connect(self.radioPluginType1, SIGNAL("toggled (bool)"), self.changePluginPolicy)
+    self.connect(self.radioPluginType2, SIGNAL("toggled (bool)"), self.changePluginPolicy)
+    if repositories.checkingOnStart():
+      self.checkUpdates.setChecked(Qt.Checked)
+    else:
+      self.checkUpdates.setChecked(Qt.Unchecked)
+    interval = repositories.checkingOnStartInterval()
+    intervals = [0,1,3,7,14,30] # days
+    if intervals.count(interval):
+      index = intervals.index(interval)
+    else:
+      index = 1
+    self.comboInterval.setCurrentIndex(index)
     self.populateMostWidgets()
 
 
   # ----------------------------------------- #
   def getAllAvailablePlugins(self):
-    """ repopulate the mPlugins dict """
+    """ fetch plugins from all repositories """
     repositories.load()
-    plugins.clear()
+    plugins.getAllInstalled()
+
     for key in repositories.allEnabled():
       repositories.requestFetching(key)
 
@@ -289,12 +310,14 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
       for key in repositories.all():
         repositories.killConnection(key)
 
-    # display error messages for every unavailable reposioty, except the case if all repositories are unavailable!
+    # display error messages for every unavailable reposioty, unless Shift pressed nor all repositories are unavailable
+    keepQuiet = QgsApplication.keyboardModifiers() == Qt.KeyboardModifiers(Qt.ShiftModifier)
     if repositories.allUnavailable() and repositories.allUnavailable() != repositories.allEnabled():
       for key in repositories.allUnavailable():
-        QMessageBox.warning(self, self.tr("QGIS Python Plugin Installer"), self.tr("Error reading repository:") + " " + key + "\n" + repositories.all()[key]["error"])
-
-    plugins.getAllInstalled()
+        if not keepQuiet:
+          QMessageBox.warning(self, self.tr("QGIS Python Plugin Installer"), self.tr("Error reading repository:") + " " + key + "\n" + repositories.all()[key]["error"])
+        if QgsApplication.keyboardModifiers() == Qt.KeyboardModifiers(Qt.ShiftModifier):
+          keepQuiet = True
 
 
   # ----------------------------------------- #
@@ -324,17 +347,35 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
           a.setToolTip(0,self.tr("This repository is disabled"))
         else:
           a.setToolTip(0,self.tr("This repository is blocked due to incompatibility with your Quantum GIS version"))
-        a.setDisabled(True)
+        for i in [0,1,2]:
+          a.setForeground(i,QBrush(QColor(Qt.gray)))
     for i in [0,1,2]:
       self.treeRepositories.resizeColumnToContents(i)
     self.comboFilter1.addItem(self.tr("orphans"))
-    # filling the status filter comboBox
+    # fill the status filter comboBox
     self.comboFilter2.clear()
     self.comboFilter2.addItem(self.tr("any status"))
     self.comboFilter2.addItem(self.tr("not installed", "plural"))
     self.comboFilter2.addItem(self.tr("installed", "plural"))
     if plugins.isThereAnythingNew():
       self.comboFilter2.addItem(self.tr("upgradeable and news"))
+    #set configuration widgets (dependent on the repository list)
+    if len(repositories.all()) == 1:
+      self.radioPluginType0.setEnabled(False)
+      self.radioPluginType1.setEnabled(False)
+      self.radioPluginType2.setEnabled(False)
+    else:
+      self.radioPluginType0.setEnabled(True)
+      self.radioPluginType1.setEnabled(True)
+      self.radioPluginType2.setEnabled(True)
+    settings = QSettings()
+    (i, ok) = settings.value(settingsGroup+"/allowedPluginType", QVariant(2)).toInt()
+    if i == 1 or len(repositories.all()) == 1:
+      self.radioPluginType0.setChecked(Qt.Checked)
+    elif i == 3:
+      self.radioPluginType2.setChecked(Qt.Checked)
+    else:
+      self.radioPluginType1.setChecked(Qt.Checked)
 
 
   # ----------------------------------------- #
@@ -362,10 +403,9 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
       return True
     else:
       for i in ["name","version_inst","version_avail","desc_repo","desc_local","author","status","repository"]:
-        item = QString(plugin[i]) #.toUpper()
+        item = QString(plugin[i])
         if item != None:
           if item.contains(self.lineFilter.text(), Qt.CaseInsensitive):
-          #if item.find(self.lineFilter.text().toUpper()) > -1:
             return True
     return False
 
@@ -461,7 +501,7 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
           a.setToolTip(4,"")
         a.setText(5,repository)
         a.setToolTip(5,p["url"])
-        # set fonts and colours
+        # set fonts and colors
         for i in [0,1,2,3,4,5]:
           if p["error"]:
             a.setForeground(i,QBrush(QColor(Qt.red)))
@@ -524,12 +564,25 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
 
 
   # ----------------------------------------- #
-  def installPlugin(self):
-    """ install currently selected plugin """
+  def installPluginClicked(self):
     if not self.treePlugins.currentItem():
       return
-    infoString = ('','')
     key = plugins.keyByUrl(self.treePlugins.currentItem().toolTip(5))
+    self.installPlugin(key)
+
+
+  # ----------------------------------------- #
+  def uninstallPluginClicked(self):
+    if not self.treePlugins.currentItem():
+      return
+    key = plugins.keyByUrl(self.treePlugins.currentItem().toolTip(5))
+    self.uninstallPlugin(key)
+
+
+# ----------------------------------------- #
+  def installPlugin(self, key):
+    """ install currently selected plugin """
+    infoString = ('','')
     plugin = plugins.all()[key]
     previousStatus = plugin["status"]
     if not plugin:
@@ -545,24 +598,42 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
     elif not QDir(QDir.cleanPath(QgsApplication.qgisSettingsDirPath() + "/python/plugins/" + key)).exists():
       infoString = (self.tr("Plugin has disappeared"), self.tr("The plugin seems to have been installed but I don't know where. Probably the plugin package contained a wrong named directory.\nPlease search the list of installed plugins. I'm nearly sure you'll find the plugin there, but I just can't determine which of them it is. It also means that I won't be able to determine if this plugin is installed and inform you about available updates. However the plugin may work. Please contact the plugin author and submit this issue."))
       QApplication.setOverrideCursor(Qt.WaitCursor)
-      self.getAllAvailablePlugins()
+      plugins.getAllInstalled()
+      plugins.rebuild()
       QApplication.restoreOverrideCursor()
     else:
-      try:
-        exec ("sys.path_importer_cache.clear()")
-        exec ("import %s" % plugin["localdir"])
-        exec ("reload (%s)" % plugin["localdir"])
-      except:
-        pass
-      plugins.updatePlugin(key, False)
+      if QGIS_14:
+        if QGIS_15: # update the list of plugins in plugin handling routines
+          updateAvailablePlugins()
+        # try to load the plugin
+        loadPlugin(plugin["localdir"])
+      else: # QGIS < 1.4
+        try:
+          exec ("sys.path_importer_cache.clear()")
+          exec ("import %s" % plugin["localdir"])
+          exec ("reload (%s)" % plugin["localdir"])
+        except:
+          pass
+      plugins.getAllInstalled()
+      plugins.rebuild()
       plugin = plugins.all()[key]
       if not plugin["error"]:
         if previousStatus in ["not installed", "new"]:
-          infoString = (self.tr("Plugin installed successfully"),
-          self.tr("Python plugin installed.\nNow you need to enable it in Plugin Manager."))
+          if QGIS_14: # plugins can be started in python from QGIS >= 1.4
+            infoString = (self.tr("Plugin installed successfully"), self.tr("Plugin installed successfully"))
+            settings = QSettings()
+            settings.setValue("/PythonPlugins/"+plugin["localdir"], QVariant(True))
+            startPlugin(plugin["localdir"])
+          else: infoString = (self.tr("Plugin installed successfully"), self.tr("Python plugin installed.\nNow you need to enable it in Plugin Manager."))
         else:
-          infoString = (self.tr("Plugin reinstalled successfully"),
-          self.tr("Python plugin reinstalled.\nYou need to restart Quantum GIS in order to reload it."))
+          if QGIS_15: # plugins can be reloaded on the fly in QGIS  >= 1.5
+            settings = QSettings()
+            if key != 'plugin_installer' and settings.value("/PythonPlugins/"+key).toBool(): # plugin will be reloaded on the fly only if currently loaded
+              infoString = (self.tr("Plugin reinstalled successfully"), self.tr("Plugin reinstalled successfully"))
+              reloadPlugin(key)
+            else:
+              infoString = (self.tr("Plugin reinstalled successfully"), self.tr("Python plugin reinstalled.\nYou need to restart Quantum GIS in order to reload it."))
+          else: infoString = (self.tr("Plugin reinstalled successfully"), self.tr("Python plugin reinstalled.\nYou need to restart Quantum GIS in order to reload it."))
       else:
         if plugin["error"] == "incompatible":
           message = self.tr("The plugin is designed for a newer version of Quantum GIS. The minimum required version is:")
@@ -577,11 +648,8 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
         dlg.exec_()
         if dlg.result():
           # revert installation
-          plugins.setPluginData(key, "status", "not installed")
-          plugins.setPluginData(key, "version_inst", "")
-          plugins.setPluginData(key, "desc_local", "")
-          plugins.setPluginData(key, "error", "")
-          plugins.setPluginData(key, "error_details", "")
+          plugins.getAllInstalled()
+          plugins.rebuild()
           pluginDir = QFileInfo(QgsApplication.qgisUserDbFilePath()).path() + "/python/plugins/" + plugin["localdir"]
           removeDir(pluginDir)
           if QDir(pluginDir).exists():
@@ -592,39 +660,62 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
               exec ("reload (%s)" % plugin["localdir"])
             except:
               pass
-            plugins.updatePlugin(key, False)
           else:
             try:
               exec ("del sys.modules[%s]" % plugin["localdir"])
             except:
               pass
-            if not plugin["repository"]:
-              plugins.remove(key)
+          plugins.getAllInstalled()
+          plugins.rebuild()
+    if plugins.all().has_key(key) and not plugins.all()[key]["status"] in ["not installed", "new"]:
+      if previousStatus in ["not installed", "new"]:
+        history.markChange(key,'A')
+      else:
+        history.markChange(key,'R')
+
     self.populatePluginTree()
     if infoString[0]:
       QMessageBox.information(self, infoString[0], infoString[1])
 
 
   # ----------------------------------------- #
-  def uninstallPlugin(self):
+  def uninstallPlugin(self,key):
     """ uninstall currently selected plugin """
-    if not self.treePlugins.currentItem():
-      return
-    key = plugins.keyByUrl(self.treePlugins.currentItem().toolTip(5))
     plugin = plugins.all()[key]
     if not plugin:
       return
-    warning = self.tr("Are you sure you want to uninstall the following plugin?") + "\n" + plugin["name"]
+    warning = self.tr("Are you sure you want to uninstall the following plugin?") + "\n(" + plugin["name"] + ")"
     if plugin["status"] == "orphan" and not plugin["error"]:
       warning += "\n\n"+self.tr("Warning: this plugin isn't available in any accessible repository!")
     if QMessageBox.warning(self, self.tr("QGIS Python Plugin Installer"), warning , QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
       return
+    # unload the plugin if it's not plugin_installer itself (otherwise, do it after removing its directory):
+    if key != "plugin_installer":
+      try:
+        unloadPlugin(key)
+      except:
+        pass
     pluginDir = QFileInfo(QgsApplication.qgisUserDbFilePath()).path() + "/python/plugins/" + plugin["localdir"]
     result = removeDir(pluginDir)
     if result:
       QMessageBox.warning(self, self.tr("Plugin uninstall failed"), result)
     else:
+      # if the uninstalled plugin is the installer itself, reload it and quit
+      if key == "plugin_installer":
+        if QGIS_15:
+          try:
+            QMessageBox.information(self, self.tr("QGIS Python Plugin Installer"), self.tr("Plugin Installer update uninstalled. Plugin Installer will now close and revert to its primary version. You can find it in the Plugins menu and continue operation."))
+            reloadPlugin(key)
+            return
+          except:
+            pass
+        else:
+          QMessageBox.information(self, self.tr("QGIS Python Plugin Installer"), self.tr("Plugin Installer update uninstalled. Please restart QGIS in order to load its primary version."))
       # safe remove
+      try:
+        unloadPlugin(plugin["localdir"])
+      except:
+        pass
       try:
         exec ("plugins[%s].unload()" % plugin["localdir"])
         exec ("del plugins[%s]" % plugin["localdir"])
@@ -634,16 +725,12 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
         exec ("del sys.modules[%s]" % plugin["localdir"])
       except:
         pass
-      if not plugin["repository"]:
-        plugins.remove(key)
-      else:
-        plugins.setPluginData(key, "status", "not installed")
-        plugins.setPluginData(key, "version_inst", "")
-        plugins.setPluginData(key, "desc_local", "")
-        plugins.setPluginData(key, "error", "")
-        plugins.setPluginData(key, "error_details", "")
+      plugins.getAllInstalled()
+      plugins.rebuild()
       self.populatePluginTree()
-      QMessageBox.information(self, self.tr("Plugin uninstalled successfully"), self.tr("Python plugin uninstalled. Note that you may need to restart Quantum GIS in order to remove it completely."))
+      if QGIS_14: QMessageBox.information(self, self.tr("Plugin uninstalled successfully"), self.tr("Plugin uninstalled successfully"))
+      else: QMessageBox.information(self, self.tr("Plugin uninstalled successfully"), self.tr("Python plugin uninstalled. Note that you may need to restart Quantum GIS in order to remove it completely."))
+      history.markChange(key,'D')
 
 
   # ----------------------------------------- #
@@ -658,13 +745,38 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
 
 
   # ----------------------------------------- #
-  def ChangeCheckingPolicy(self,policy):
-    if policy == Qt.Checked:
+  def changeCheckingPolicy(self,policy):
+    """ the Checking On Start checkbox has been clicked """
+    if policy:
       repositories.setCheckingOnStart(True)
     else:
       repositories.setCheckingOnStart(False)
 
 
+  # ----------------------------------------- #
+  def changeCheckingInterval(self,interval):
+    """ the Checking on start interval combobox has been clicked """
+    intervals = [0,1,3,7,14,30]
+    repositories.setCheckingOnStartInterval(intervals[interval])
+
+
+  # ----------------------------------------- #
+  def changePluginPolicy(self, state):
+    """ one of the plugin type radiobuttons has been clicked """
+    if not state: # radio button released
+      return
+    if self.radioPluginType0.isChecked():
+      i = 1
+    elif self.radioPluginType1.isChecked():
+      i = 2
+    else:
+      i = 3
+    settings = QSettings()
+    settings.setValue(settingsGroup+"/allowedPluginType", QVariant(i))
+    plugins.rebuild()
+    self.populatePluginTree()
+
+
   # ----------------------------------------- #
   def addKnownRepositories(self):
     """ update list of known repositories - in the future it will be replaced with an online fetching """
@@ -674,6 +786,7 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
       # refresh lists and populate widgets
       QApplication.setOverrideCursor(Qt.WaitCursor)
       self.getAllAvailablePlugins()
+      plugins.rebuild()
       self.populateMostWidgets()
       self.populatePluginTree()
       QApplication.restoreOverrideCursor()
@@ -687,13 +800,13 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
     if not dlg.exec_():
       return
     for i in repositories.all().values():
-      if dlg.editURL.text() == i["url"]:
+      if dlg.editURL.text().trimmed() == i["url"]:
         QMessageBox.warning(self, self.tr("QGIS Python Plugin Installer"), self.tr("Unable to add another repository with the same URL!"))
         return
     settings = QSettings()
     settings.beginGroup(self.reposGroup)
     reposName = dlg.editName.text()
-    reposURL = dlg.editURL.text()
+    reposURL = dlg.editURL.text().trimmed()
     if repositories.all().has_key(reposName):
       reposName = reposName + "(2)"
     # add to settings
@@ -701,7 +814,9 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
     settings.setValue(reposName+"/enabled", QVariant(bool(dlg.checkBoxEnabled.checkState())))
     # refresh lists and populate widgets
     QApplication.setOverrideCursor(Qt.WaitCursor)
+    plugins.removeRepository(reposName)
     self.getAllAvailablePlugins()
+    plugins.rebuild()
     self.populateMostWidgets()
     self.populatePluginTree()
     QApplication.restoreOverrideCursor()
@@ -729,7 +844,7 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
     if not dlg.exec_():
       return # nothing to do if cancelled
     for i in repositories.all().values():
-      if dlg.editURL.text() == i["url"] and dlg.editURL.text() != repositories.all()[reposName]["url"]:
+      if dlg.editURL.text().trimmed() == i["url"] and dlg.editURL.text().trimmed() != repositories.all()[reposName]["url"]:
         QMessageBox.warning(self, self.tr("QGIS Python Plugin Installer"), self.tr("Unable to add another repository with the same URL!"))
         return
     # delete old repo from QSettings and create new one
@@ -739,15 +854,17 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
     newName = dlg.editName.text()
     if repositories.all().has_key(newName) and newName != reposName:
       newName = newName + "(2)"
-    settings.setValue(newName+"/url", QVariant(dlg.editURL.text()))
+    settings.setValue(newName+"/url", QVariant(dlg.editURL.text().trimmed()))
     settings.setValue(newName+"/enabled", QVariant(bool(dlg.checkBoxEnabled.checkState())))
-    if dlg.editURL.text() == repositories.all()[reposName]["url"] and dlg.checkBoxEnabled.checkState() == checkState[repositories.all()[reposName]["enabled"]]:
+    if dlg.editURL.text().trimmed() == repositories.all()[reposName]["url"] and dlg.checkBoxEnabled.checkState() == checkState[repositories.all()[reposName]["enabled"]]:
       repositories.rename(reposName, newName)
       self.populateMostWidgets()
       return # nothing else to do if only repository name was changed
     # refresh lists and populate widgets
     QApplication.setOverrideCursor(Qt.WaitCursor)
+    plugins.removeRepository(reposName)
     self.getAllAvailablePlugins()
+    plugins.rebuild()
     self.populateMostWidgets()
     self.populatePluginTree()
     QApplication.restoreOverrideCursor()
@@ -763,16 +880,21 @@ class QgsPluginInstallerDialog(QDialog, Ui_QgsPluginInstallerDialogBase):
     if QMessageBox.warning(self, self.tr("QGIS Python Plugin Installer"), warning , QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
       return
     reposName = current.text(1)
+    # delete from the settings, refresh data and repopulate all the widgets
     settings = QSettings()
     settings.beginGroup(self.reposGroup)
-    # delete from settings
     settings.remove(reposName)
-    # refresh lists and populate widgets
-    QApplication.setOverrideCursor(Qt.WaitCursor)
-    self.getAllAvailablePlugins()
+    repositories.remove(reposName)
+    plugins.removeRepository(reposName)
+    plugins.rebuild()
     self.populateMostWidgets()
     self.populatePluginTree()
-    QApplication.restoreOverrideCursor()
+
+
+  # ----------------------------------------- #
+  def runHelp(self):
+    """ open the context help browser """
+    QgsContextHelp.run("QgsPluginInstallerDialog")
 
 
   # ----------------------------------------- #
